@@ -23,6 +23,7 @@
 
 #include <stdio.h>
 #include <string.h>
+#include <stdlib.h>
 #include <gensio/gensio.h>
 #include <gensio/gensio_list.h>
 
@@ -61,7 +62,7 @@ shutdown_done(struct gensio_accepter *acc, void *shutdown_data)
 {
     struct accinfo *ai = shutdown_data;
 
-    ai->o->wake(ai->waiter);
+    gensio_os_funcs_wake(ai->o, ai->waiter);
 }
 
 static void
@@ -88,7 +89,7 @@ close_done(struct gensio *io, void *close_data)
 
     gensio_free(io);
     gensio_list_rm(&ai->ios, &ii->link);
-    ai->o->free(ai->o, ii);
+    free(ii);
     check_shutdown(ai);
 }
 
@@ -137,7 +138,7 @@ start_close(struct ioinfo *ii)
 	/* Should be impossible, but just in case... */
 	fprintf(stderr, "Error closing io: %s\n", gensio_err_to_str(rv));
 	gensio_free(ii->io);
-	ii->ai->o->free(ii->ai->o, ii);
+	free(ii);
     }
 }
 
@@ -190,7 +191,7 @@ io_event(struct gensio *io, void *user_data, int event, int err,
 	    if (err != GE_REMCLOSE)
 		fprintf(stderr, "Error from io: %s\n", gensio_err_to_str(err));
 	    start_close(ii);
-	    return 0; 
+	    return 0;
 	}
 
 	len = *buflen;
@@ -245,9 +246,9 @@ io_event(struct gensio *io, void *user_data, int event, int err,
 	    }
 	}
 	if (ii->outbuf_len == 0) {
+	    gensio_set_write_callback_enable(ii->io, false);
 	    if (ii->close_on_write && !ii->closing)
 		start_close(ii);
-	    gensio_set_write_callback_enable(ii->io, false);
 	}
 	return 0;
 
@@ -279,7 +280,7 @@ io_acc_event(struct gensio_accepter *accepter, void *user_data,
 	return 0;
     }
 
-    ii = ai->o->zalloc(ai->o, sizeof(*ii));
+    ii = calloc(1, sizeof(*ii));
     if (!ii) {
 	fprintf(stderr, "Could not allocate info for new io\n");
 	gensio_free(data);
@@ -291,7 +292,7 @@ io_acc_event(struct gensio_accepter *accepter, void *user_data,
     gensio_set_callback(ii->io, io_event, ii);
     gensio_set_read_callback_enable(ii->io, true);
     add_output_buf(ii, "Ready\n");
-    
+
     return 0;
 }
 
@@ -300,7 +301,7 @@ thread2(void *data)
 {
     struct accinfo *ai = data;
 
-    ai->o->wait(ai->thread2_waiter, 1, NULL);
+    gensio_os_funcs_wait(ai->o, ai->thread2_waiter, 1, NULL);
 }
 
 int
@@ -325,7 +326,7 @@ main(int argc, char *argv[])
 		gensio_err_to_str(rv));
 	return 1;
     }
-    ai.o->vlog = do_vlog;
+    gensio_os_funcs_set_vlog(ai.o, do_vlog);
 
     rv = gensio_os_proc_setup(ai.o, &proc_data);
     if (rv) {
@@ -334,14 +335,14 @@ main(int argc, char *argv[])
 	return 1;
     }
 
-    ai.waiter = ai.o->alloc_waiter(ai.o);
+    ai.waiter = gensio_os_funcs_alloc_waiter(ai.o);
     if (!ai.waiter) {
 	rv = GE_NOMEM;
 	fprintf(stderr, "Could not waiter, out of memory\n");
 	goto out_err;
     }
 
-    ai.thread2_waiter = ai.o->alloc_waiter(ai.o);
+    ai.thread2_waiter = gensio_os_funcs_alloc_waiter(ai.o);
     if (!ai.thread2_waiter) {
 	rv = GE_NOMEM;
 	fprintf(stderr, "Could not thread2 waiter, out of memory\n");
@@ -371,21 +372,21 @@ main(int argc, char *argv[])
 	goto out_err;
     }
 
-    rv = ai.o->wait(ai.waiter, 1, NULL);
+    rv = gensio_os_funcs_wait(ai.o, ai.waiter, 1, NULL);
 
  out_err:
     if (tid2) {
-	ai.o->wake(ai.thread2_waiter);
+	gensio_os_funcs_wake(ai.o, ai.thread2_waiter);
 	gensio_os_wait_thread(tid2);
     }
     if (ai.acc)
 	gensio_acc_free(ai.acc);
     if (ai.waiter)
-	ai.o->free_waiter(ai.waiter);
+	gensio_os_funcs_free_waiter(ai.o, ai.waiter);
     if (ai.thread2_waiter)
-	ai.o->free_waiter(ai.thread2_waiter);
+	gensio_os_funcs_free_waiter(ai.o, ai.thread2_waiter);
     gensio_os_proc_cleanup(proc_data);
-    ai.o->free_funcs(ai.o);
+    gensio_os_funcs_free(ai.o);
 
     return !!rv;
 }
