@@ -46,6 +46,7 @@
 #include <gensio/gensio_osops.h>
 #include <gensio/gensio_osops_addrinfo.h>
 #include <gensio/gensio_osops_stdsock.h>
+#include <gensio/argvutils.h>
 
 #include <glib.h>
 
@@ -59,6 +60,7 @@
 #include <fcntl.h>
 #include <sys/wait.h>
 #include <pthread.h>
+#include <sys/ioctl.h>
 #endif
 
 struct gensio_data
@@ -157,6 +159,7 @@ struct gensio_iod_glib {
     unsigned int clear_count;
     enum { CL_NOT_CALLED, CL_CALLED, CL_DONE } close_state;
 
+    int orig_fd;
     int fd;
     enum gensio_iod_type type;
     void *sockinfo;
@@ -182,6 +185,12 @@ struct gensio_iod_glib {
     bool read_enabled;
     bool write_enabled;
     bool in_handler;
+
+    /* For GENSIO_IOD_PTY */
+    const char **argv;
+    const char **env;
+    char *start_dir;
+    int pid;
 };
 
 #define i_to_glib(i) gensio_container_of(i, struct gensio_iod_glib, r);
@@ -1086,11 +1095,12 @@ gensio_glib_wake(struct gensio_waiter *w)
 
 static int
 gensio_glib_add_iod(struct gensio_os_funcs *o, enum gensio_iod_type type,
-		    intptr_t fd, struct gensio_iod **riod)
+		    intptr_t ofd, struct gensio_iod **riod)
 {
     struct gensio_iod_glib *iod;
     bool closefd = false;
     int err = GE_NOMEM;
+    intptr_t fd = ofd;
 
 #ifndef _WIN32
     if (type == GENSIO_IOD_CONSOLE) {
@@ -1103,6 +1113,13 @@ gensio_glib_add_iod(struct gensio_os_funcs *o, enum gensio_iod_type type,
 	if (fd == -1)
 	    return gensio_os_err_to_err(o, errno);
 	closefd = true;
+    } else if (type == GENSIO_IOD_PTY) {
+	int ufd;
+	err = gensio_unix_pty_alloc(o, &ufd);
+	if (err)
+	    return err;
+	fd = ufd;
+	closefd = true;
     }
 #endif
 
@@ -1114,6 +1131,7 @@ gensio_glib_add_iod(struct gensio_os_funcs *o, enum gensio_iod_type type,
     }
     iod->r.f = o;
     iod->fd = fd;
+    iod->orig_fd = ofd;
     if (type == GENSIO_IOD_STDIO) {
 #ifndef _WIN32
 	struct stat statb;
@@ -1134,6 +1152,8 @@ gensio_glib_add_iod(struct gensio_os_funcs *o, enum gensio_iod_type type,
 	    err = GE_INVAL;
 	    goto out_err;
 	}
+    } else if (type == GENSIO_IOD_PTY) {
+	iod->pid = -1;
 #endif
     }
     iod->type = type;
@@ -1183,13 +1203,20 @@ gensio_glib_add_iod(struct gensio_os_funcs *o, enum gensio_iod_type type,
 static void
 gensio_glib_release_iod(struct gensio_iod *iiod)
 {
+    struct gensio_os_funcs *o = iiod->f;
     struct gensio_iod_glib *iod = i_to_glib(iiod);
 
     assert(!iod->handlers_set);
     g_mutex_clear(&iod->lock);
     if (iod->type == GENSIO_IOD_FILE)
-	iod->r.f->free_runner(iod->runner);
-    iiod->f->free(iiod->f, iod);
+	o->free_runner(iod->runner);
+    if (iod->type == GENSIO_IOD_PTY) {
+	if (iod->argv)
+	    gensio_argv_free(o, iod->argv);
+	if (iod->env)
+	    gensio_argv_free(o, iod->env);
+    }
+    o->free(o, iod);
 }
 
 static int
@@ -1208,6 +1235,95 @@ gensio_glib_iod_get_fd(struct gensio_iod *iiod)
     return iod->fd;
 }
 
+static int
+gensio_glib_pty_control(struct gensio_iod_glib *iod, int op, bool get,
+			intptr_t val)
+{
+    struct gensio_os_funcs *o = iod->r.f;
+    int err = 0;
+    const char **nargv;
+
+    if (get) {
+	if (op == GENSIO_IOD_CONTROL_PID) {
+	    if (iod->pid == -1)
+		return GE_NOTREADY;
+	    *((intptr_t *) val) = iod->pid;
+	    return 0;
+	}
+	return GE_NOTSUP;
+    }
+
+    switch (op) {
+    case GENSIO_IOD_CONTROL_ARGV:
+	err = gensio_argv_copy(o, (const char **) val, NULL, &nargv);
+	if (err)
+	    return err;
+	if (iod->argv)
+	    gensio_argv_free(o, iod->argv);
+	iod->argv = nargv;
+	return 0;
+
+    case GENSIO_IOD_CONTROL_ENV:
+	err = gensio_argv_copy(o, (const char **) val, NULL, &nargv);
+	if (err)
+	    return err;
+	if (iod->env)
+	    gensio_argv_free(o, iod->env);
+	iod->env = nargv;
+	return 0;
+
+    case GENSIO_IOD_CONTROL_START:
+#ifdef _WIN32
+	return GE_NOTSUP;
+#else
+	return gensio_unix_pty_start(o, iod->fd, iod->argv,
+				     iod->env, iod->start_dir, &iod->pid);
+#endif
+
+    case GENSIO_IOD_CONTROL_STOP:
+#ifdef _WIN32
+	return GE_NOTSUP;
+#else
+	if (iod->fd != -1) {
+	    close(iod->fd);
+	    iod->fd = -1;
+	}
+	return 0;
+#endif
+
+    case GENSIO_IOD_CONTROL_WIN_SIZE: {
+	struct winsize win;
+	struct gensio_winsize *gwin = (struct gensio_winsize *) val;
+
+	win.ws_row = gwin->ws_row;
+	win.ws_col = gwin->ws_col;
+	win.ws_xpixel = gwin->ws_xpixel;
+	win.ws_ypixel = gwin->ws_ypixel;
+	if (ioctl(iod->fd, TIOCSWINSZ, &win) == -1)
+	    err = gensio_os_err_to_err(o, errno);
+	return err;
+    }
+
+    case GENSIO_IOD_CONTROL_START_DIR: {
+	char *dir = (char *) val;
+
+	if (dir) {
+	    dir = gensio_strdup(o, dir);
+	    if (!dir)
+		return GE_NOMEM;
+	}
+
+	if (iod->start_dir)
+	    o->free(o, iod->start_dir);
+	iod->start_dir = dir;
+	return 0;
+    }
+
+    default:
+	return GE_NOTSUP;
+    }
+}
+
 static int
 gensio_glib_iod_control(struct gensio_iod *iiod, int op, bool get, intptr_t val)
 {
@@ -1225,6 +1341,9 @@ gensio_glib_iod_control(struct gensio_iod *iiod, int op, bool get, intptr_t val)
 	return 0;
     }
 
+    if (iod->type == GENSIO_IOD_PTY)
+	return gensio_glib_pty_control(iod, op, get, val);
+
     if (iod->type != GENSIO_IOD_DEV)
 	return GE_NOTSUP;
 
@@ -1302,14 +1421,16 @@ gensio_glib_close(struct gensio_iod **iodp)
 #ifdef _WIN32
 	CloseHandle(iod->h);
 #else
-	err = close(iod->fd);
-	if (err == -1)
-	    err = gensio_os_err_to_err(o, errno);
+	if (iod->fd != -1) {
+	    err = close(iod->fd);
+	    if (err == -1)
+		err = gensio_os_err_to_err(o, errno);
 #endif
 #ifdef ENABLE_INTERNAL_TRACE
 	/* Close should never fail, but don't crash in production builds. */
-	assert(err == 0);
+	    assert(err == 0);
 #endif
+	}
     }
     o->release_iod(iiod);
     *iodp = NULL;
@@ -1503,7 +1624,7 @@ gensio_glib_makeraw(struct gensio_iod *iiod)
 
     return GE_NOTSUP;
 #else
-    if (iod->fd == 1 || iod->fd == 2 || iod->type == GENSIO_IOD_FILE)
+    if (iod->orig_fd == 1 || iod->orig_fd == 2 || iod->type == GENSIO_IOD_FILE)
 	/* Only set this for stdin or other files. */
 	return 0;
 
@@ -1608,12 +1729,13 @@ generic_close(intptr_t fd)
 
 static int
 gensio_glib_exec_subprog(struct gensio_os_funcs *o,
-			const char *argv[], const char **env,
-			bool stderr_to_stdout,
-			intptr_t *rpid,
-			struct gensio_iod **rstdin,
-			struct gensio_iod **rstdout,
-			struct gensio_iod **rstderr)
+			 const char *argv[], const char **env,
+			 const char *start_dir,
+			 unsigned int flags,
+			 intptr_t *rpid,
+			 struct gensio_iod **rstdin,
+			 struct gensio_iod **rstdout,
+			 struct gensio_iod **rstderr)
 {
     int err;
     struct gensio_iod *stdiniod = NULL, *stdoutiod = NULL, *stderriod = NULL;
@@ -1623,7 +1745,7 @@ gensio_glib_exec_subprog(struct gensio_os_funcs *o,
 #ifdef _WIN32
     HANDLE winfd, woutfd, werrfd = NULL, wpid;
 
-    err = gensio_win_do_exec(o, argv, env, stderr_to_stdout, &wpid, &winfd,
+    err = gensio_win_do_exec(o, argv, env, start_dir, flags, &wpid, &winfd,
 			     &woutfd, rstderr ? &werrfd : NULL);
     if (err)
 	return err;
@@ -1635,7 +1757,7 @@ gensio_glib_exec_subprog(struct gensio_os_funcs *o,
     int uinfd = -1, uoutfd = -1, uerrfd = -1;
     int upid = -1;
 
-    err = gensio_unix_do_exec(o, argv, env, stderr_to_stdout, &upid, &uinfd,
+    err = gensio_unix_do_exec(o, argv, env, start_dir, flags, &upid, &uinfd,
 			      &uoutfd, rstderr ? &uerrfd : NULL);
     if (err)
 	return err;
