@@ -26,8 +26,8 @@ typedef int (*gensio_ll_filter_data_handler)(void *cb_data,
 					     const char *const *auxdata);
 
 /*
- * The filter has some asynchronously generated data that it needs
- * to send, tell the gensio base to recalculate its enables.
+ * The filter has some asynchronously generated data that it needs to
+ * send, tell the gensio base to recalculate its enables.
  */
 #define GENSIO_FILTER_CB_OUTPUT_READY	1
 
@@ -55,6 +55,17 @@ struct gensio_filter_cb_control_data {
 };
 #define GENSIO_FILTER_CB_CONTROL	4
 
+/*
+ * During open, check_open_done needs to be called.
+ */
+#define GENSIO_FILTER_CB_OPEN_DONE	5
+
+/*
+ * An asynchronous event has happend where the filter is ready for
+ * lower-level input again.
+ */
+#define GENSIO_FILTER_CB_INPUT_READY	6
+
 typedef int (*gensio_filter_cb)(void *cb_data, int func, void *data);
 
 
@@ -224,9 +235,9 @@ int gensio_filter_control(struct gensio_filter *filter, bool get,
  *
  * &val => data (pointer to a bool)
  */
-#define GENSIO_FILTER_FUNC_LL_CAN_WRITE		16
+#define GENSIO_FILTER_FUNC_UL_CAN_WRITE		16
 GENSIO_DLL_PUBLIC
-bool gensio_filter_ll_can_write(struct gensio_filter *filter);
+bool gensio_filter_ul_can_write(struct gensio_filter *filter);
 
 /*
  * Does the filter have write data queued?  This is different from
