@@ -9,7 +9,15 @@
  * This allows a lot of pthread calls to be dummied out if pthreads is
  * disabled.
  */
-#ifdef USE_PTHREADS
+#ifdef _WIN32
+#include <windows.h>
+#define lock_type SRWLOCK
+#define LOCK_INIT(l) InitializeSRWLock(l)
+#define LOCK_DESTROY(l) do {} while(0)
+#define LOCK(l) AcquireSRWLockExclusive(l)
+#define UNLOCK(l) ReleaseSRWLockExclusive(l)
+#define LOCK_INITIALIZER SRWLOCK_INIT
+#elif defined(USE_PTHREADS)
 #include <pthread.h>
 #define lock_type pthread_mutex_t
 #define LOCK_INIT(l) pthread_mutex_init(l, NULL)
