@@ -12,6 +12,7 @@
 #ifdef _WIN32
 #include <winsock2.h>
 #include <ws2tcpip.h>
+#include <netioapi.h>
 #else
 #include <errno.h>
 #include <sys/types.h>
@@ -19,6 +20,7 @@
 #include <netinet/in.h>
 #include <netdb.h>
 #include <arpa/inet.h>
+#include <net/if.h>
 #endif
 #include <assert.h>
 #include <stdlib.h>
@@ -188,10 +190,18 @@ gensio_addr_addrinfo_create(struct gensio_os_funcs *o,
 
     case GENSIO_NETTYPE_IPV6:
 #ifdef AF_INET6
-	if (len != sizeof(struct in6_addr))
+	if (len == sizeof(struct in6_addr)) {
+	    memcpy(&s6.sin6_addr, iaddr, len);
+	} else if (len == sizeof(s6)) {
+	    /* Full sockaddr_in6, so we can get scope. */
+	    const struct sockaddr_in6 *is6 = iaddr;
+
+	    memcpy(&s6.sin6_addr, &is6->sin6_addr, sizeof(struct in6_addr));
+	    s6.sin6_scope_id = is6->sin6_scope_id;
+	} else {
 	    return GE_INVAL;
+	}
 	s6.sin6_port = htons(port);
-	memcpy(&s6.sin6_addr, iaddr, len);
 	s = (struct sockaddr *) &s6;
 	slen = sizeof(s6);
 	break;
@@ -399,10 +409,17 @@ gensio_sockaddr_to_str(const struct sockaddr *addr, int flags,
     } else if (addr->sa_family == AF_INET6) {
 	struct sockaddr_in6 *a6 = (struct sockaddr_in6 *) addr;
 	char ibuf[INET6_ADDRSTRLEN];
+	char ifbuf[IF_NAMESIZE + 1];
 
-	gensio_pos_snprintf(buf, buflen, pos, "%s,%s,%d",
+	if (IN6_IS_ADDR_LINKLOCAL(&a6->sin6_addr) &&
+		if_indextoname(a6->sin6_scope_id, &(ifbuf[1])) != NULL)
+	    ifbuf[0] = '%';
+	else
+	    ifbuf[0] = '\0';
+	gensio_pos_snprintf(buf, buflen, pos, "%s,%s%s,%d",
 			flags & AI_V4MAPPED ? "ipv6n4" : "ipv6",
 			inet_ntop(AF_INET6, &a6->sin6_addr, ibuf, sizeof(ibuf)),
+			ifbuf,
 			ntohs(a6->sin6_port));
 #endif
 #if HAVE_UNIX
