@@ -385,10 +385,10 @@ filter_io_err(struct basen_data *ndata, int err)
 }
 
 static bool
-filter_ll_can_write(struct basen_data *ndata)
+filter_ul_can_write(struct basen_data *ndata)
 {
     if (ndata->filter)
-	return gensio_filter_ll_can_write(ndata->filter);
+	return gensio_filter_ul_can_write(ndata->filter);
     return ndata->ll_can_write;
 }
 
@@ -780,7 +780,8 @@ basen_read_data_handler(void *cb_data,
     basen_unlock(ndata);
 
  out:
-    *rcount = count;
+    if (rcount)
+	*rcount = count;
     return err;
 }
 
@@ -1021,7 +1022,7 @@ basen_deferred_op(struct gensio_runner *runner, void *cbdata)
 	    goto skip_write;
 	ndata->in_xmit_ready = true;
 	while (basen_in_write_callbackable_state(ndata) &&
-	       (filter_ll_can_write(ndata) || ndata->ll_err)
+	       (filter_ul_can_write(ndata) || ndata->ll_err)
 	       && ndata->xmit_enabled) {
 	    basen_unlock(ndata);
 	    err = gensio_cb(ndata->io, GENSIO_EVENT_WRITE_READY, 0, NULL,
@@ -1162,7 +1163,7 @@ basen_open(struct basen_data *ndata, gensio_done_err open_done, void *open_data)
 	ndata->deferred_close = false;
 	ndata->read_enabled = false;
 	ndata->xmit_enabled = false;
-	ndata->ll_can_write = true;
+	ndata->ll_can_write = false;
 	ndata->timer_start_pending = false;
 	ndata->close_requested = false;
 	ndata->ll_want_close = false;
@@ -1439,7 +1440,8 @@ basen_set_read_callback_enable(struct basen_data *ndata, bool enabled)
     if (ndata->deferred_op_pending && enabled) {
 	/* Nothing to do, let the read/open handling wake things up. */
 	ndata->deferred_read = true;
-    } else if (enabled && (read_pending || ndata->ll_err)) {
+    } else if (enabled && (read_pending || ndata->ll_err) &&
+	       ndata->state == BASEN_OPEN) {
 	ndata->deferred_read = true;
 	basen_sched_deferred_op(ndata);
     } else {
@@ -1465,7 +1467,7 @@ basen_set_write_callback_enable(struct basen_data *ndata, bool enabled)
     if (!basen_in_write_callbackable_state(ndata))
 	goto out_unlock;
     ndata->xmit_enabled = enabled;
-    if (enabled && (filter_ll_can_write(ndata) || ndata->ll_err)) {
+    if (enabled && (filter_ul_can_write(ndata) || ndata->ll_err)) {
 	/* We can write, schedule the callback as a deferred op. */
 	ndata->deferred_write = true;
 	basen_sched_deferred_op(ndata);
@@ -1585,14 +1587,15 @@ basen_ll_read(void *cb_data, int readerr,
 	goto out_finish;
     }
 
-    if (ndata->in_read) {
-	/* Currently in a deferred read, just let that handle it. */
-	ll_set_read_callback_enable(ndata, false);
-	goto out_unlock;
-    }
-
     while (buflen > 0 &&
 	   (ndata->read_enabled || filter_ll_read_needed(ndata))) {
+
+	if (ndata->in_read) {
+	    /* Currently in a deferred read, just let that handle it. */
+	    ll_set_read_callback_enable(ndata, false);
+	    goto out_unlock;
+	}
+
 	ndata->in_read = true;
 	do {
 	    gensiods wrlen = 0;
@@ -1634,7 +1637,7 @@ basen_ll_read(void *cb_data, int readerr,
 	basen_check_open_close_ops(ndata);
 
 	while (basen_in_write_callbackable_state(ndata) &&
-	       (filter_ll_can_write(ndata) || ndata->ll_err)
+	       (filter_ul_can_write(ndata) || ndata->ll_err)
 	       && ndata->xmit_enabled) {
 	    basen_unlock(ndata);
 	    err = gensio_cb(ndata->io, GENSIO_EVENT_WRITE_READY, 0, NULL,
@@ -1697,7 +1700,7 @@ basen_ll_write_ready(void *cb_data)
     basen_check_open_close_ops(ndata);
 
     while (basen_in_write_callbackable_state(ndata) &&
-	   (filter_ll_can_write(ndata) || ndata->ll_err)
+	   (filter_ul_can_write(ndata) || ndata->ll_err)
 	   && ndata->xmit_enabled) {
 	basen_unlock(ndata);
 	err = gensio_cb(ndata->io, GENSIO_EVENT_WRITE_READY, 0, NULL, 0, NULL);
@@ -1748,6 +1751,8 @@ basen_output_ready(void *cb_data)
 {
     struct basen_data *ndata = cb_data;
 
+    /* Force the lower-level callback to be enabled. */
+    ndata->ll_can_write = false;
     ll_set_write_callback_enable(ndata, true);
 }
 
@@ -1782,6 +1787,29 @@ basen_call_child_control_op(struct basen_data *ndata,
 			  ctrl->data, ctrl->datalen);
 }
 
+static void
+basen_filter_open_done(void *cb_data)
+{
+    struct basen_data *ndata = cb_data;
+
+    basen_lock(ndata);
+    if (ndata->state == BASEN_IN_FILTER_OPEN) {
+	basen_filter_try_connect_finish(ndata, false);
+	basen_set_ll_enables(ndata);
+    }
+    basen_unlock(ndata);
+}
+
+static void
+basen_filter_input_ready(void *cb_data)
+{
+    struct basen_data *ndata = cb_data;
+
+    basen_lock(ndata);
+    basen_set_ll_enables(ndata);
+    basen_unlock(ndata);
+}
+
 static int
 gensio_base_filter_cb(void *cb_data, int op, void *data)
 {
@@ -1802,6 +1830,14 @@ gensio_base_filter_cb(void *cb_data, int op, void *data)
 	basen_call_child_control_op(cb_data, data);
 	return 0;
 
+    case GENSIO_FILTER_CB_OPEN_DONE:
+	basen_filter_open_done(cb_data);
+	return 0;
+
+    case GENSIO_FILTER_CB_INPUT_READY:
+	basen_filter_input_ready(cb_data);
+	return 0;
+
     default:
 	return GE_NOTSUP;
     }
@@ -1970,13 +2006,13 @@ gensio_filter_ll_write_pending(struct gensio_filter *filter)
 }
 
 bool
-gensio_filter_ll_can_write(struct gensio_filter *filter)
+gensio_filter_ul_can_write(struct gensio_filter *filter)
 {
     bool val = true;
     int err;
 
     /* If not implemented, this will just be ignored. */
-    err = filter->func(filter, GENSIO_FILTER_FUNC_LL_CAN_WRITE,
+    err = filter->func(filter, GENSIO_FILTER_FUNC_UL_CAN_WRITE,
 		       NULL, &val, NULL, NULL, NULL, 0, NULL);
     if (err)
 	return filter->ndata->ll_can_write;
