@@ -7,11 +7,8 @@
 
 #include "config.h"
 
-#include <gensio/gensio_builtins.h>
 #include <gensio/gensio_err.h>
 
-#if HAVE_OPENSSL
-
 #include <stdlib.h>
 #include <limits.h>
 #include <string.h>
@@ -29,7 +26,7 @@
 
 #include "gensio_filter_certauth.h"
 
-int
+static int
 certauth_gensio_alloc(struct gensio *child, const char *const args[],
 		      struct gensio_os_funcs *o,
 		      gensio_event cb, void *user_data,
@@ -86,7 +83,7 @@ certauth_gensio_alloc(struct gensio *child, const char *const args[],
     return 0;
 }
 
-int
+static int
 str_to_certauth_gensio(const char *str, const char * const args[],
 		       struct gensio_os_funcs *o,
 		       gensio_event cb, void *user_data,
@@ -240,7 +237,7 @@ gensio_gensio_acc_certauth_cb(void *acc_data, int op, void *data1, void *data2,
     }
 }
 
-int
+static int
 certauth_gensio_accepter_alloc(struct gensio_accepter *child,
 			       const char * const args[],
 			       struct gensio_os_funcs *o,
@@ -282,7 +279,7 @@ certauth_gensio_accepter_alloc(struct gensio_accepter *child,
     return err;
 }
 
-int
+static int
 str_to_certauth_gensio_accepter(const char *str, const char * const args[],
 				struct gensio_os_funcs *o,
 				gensio_accepter_event cb,
@@ -302,43 +299,19 @@ str_to_certauth_gensio_accepter(const char *str, const char * const args[],
     return err;
 }
 
-#else /* HAVE_OPENSSL */
 int
-certauth_gensio_alloc(struct gensio *child, const char * const args[],
-		      struct gensio_os_funcs *o,
-		      gensio_event cb, void *user_data,
-		      struct gensio **net)
+gensio_init_certauth(struct gensio_os_funcs *o)
 {
-    return GE_NOTSUP;
-}
+    int rv;
 
-int
-str_to_certauth_gensio(const char *str, const char * const args[],
-		  struct gensio_os_funcs *o,
-		  gensio_event cb, void *user_data,
-		  struct gensio **new_gensio)
-{
-    return GE_NOTSUP;
+    rv = register_filter_gensio(o, "certauth",
+				str_to_certauth_gensio, certauth_gensio_alloc);
+    if (rv)
+	return rv;
+    rv = register_filter_gensio_accepter(o, "certauth",
+					 str_to_certauth_gensio_accepter,
+					 certauth_gensio_accepter_alloc);
+    if (rv)
+	return rv;
+    return 0;
 }
-
-int
-certauth_gensio_accepter_alloc(struct gensio_accepter *child,
-			       const char * const args[],
-			       struct gensio_os_funcs *o,
-			       gensio_accepter_event cb, void *user_data,
-			       struct gensio_accepter **accepter)
-{
-    return GE_NOTSUP;
-}
-
-int
-str_to_certauth_gensio_accepter(const char *str, const char * const args[],
-				struct gensio_os_funcs *o,
-				gensio_accepter_event cb,
-				void *user_data,
-				struct gensio_accepter **acc)
-{
-    return GE_NOTSUP;
-}
-
-#endif /* HAVE_OPENSSL */
