@@ -10,10 +10,10 @@
 #include "config.h"
 #include <assert.h>
 #include <gensio/gensio.h>
+#include <gensio/gensio_time.h>
 #include <gensio/gensio_os_funcs.h>
 #include <gensio/gensio_class.h>
 #include <gensio/gensio_base.h>
-#include <gensio/gensio_builtins.h>
 
 enum conaccn_state {
     CONACCN_CLOSED,
@@ -248,7 +248,7 @@ struct conaccna_data {
     struct conaccn_data *ndata;
 
     struct gensio_timer *retry_timer;
-    unsigned int retry_time;
+    struct gensio_time retry_time;
 
     bool deferred_op_pending;
     struct gensio_runner *deferred_runner;
@@ -486,7 +486,7 @@ conaccn_finish_close(struct conaccn_data *ndata)
 	    break;
 
 	case CONACCNA_READY:
-	    if (nadata->retry_time)
+	    if (!gensio_time_is_zero(nadata->retry_time))
 		start_retry(nadata);
 	    else
 		conacc_start(nadata);
@@ -585,7 +585,7 @@ conaccn_disable(struct conaccn_data *ndata)
     if (nadata) {
 	conaccna_lock(nadata);
 	nadata->ndata = NULL;
-	if (nadata->retry_time)
+	if (!gensio_time_is_zero(nadata->retry_time))
 	    start_retry(nadata);
 	else
 	    conacc_start(nadata);
@@ -644,13 +644,10 @@ static void
 start_retry(struct conaccna_data *nadata)
 {
     struct gensio_os_funcs *o = nadata->o;
-    gensio_time timeout = { nadata->retry_time / 1000,
-			    nadata->retry_time % 1000 };
-    int rv;
 
     nadata->state = CONACCNA_WAITING_RETRY;
-    rv = o->start_timer(nadata->retry_timer, &timeout);
-    assert(rv == 0);
+    if (o->start_timer(nadata->retry_timer, &nadata->retry_time) != 0)
+	assert(0);
     conaccna_ref(nadata);
 }
 
@@ -694,7 +691,7 @@ conaccn_open_done(struct gensio *io, int err, void *open_data)
 
     case CONACCNA_OPENING:
 	if (err) {
-	    if (nadata->retry_time) {
+	    if (!gensio_time_is_zero(nadata->retry_time)) {
 		start_retry(nadata);
 	    } else {
 		nadata->con_err = err;
@@ -772,7 +769,7 @@ conacc_start(struct conaccna_data *nadata)
  out_err:
     conaccn_finish_free(ndata);
  out_err_nofree:
-    if (nadata->retry_time) {
+    if (!gensio_time_is_zero(nadata->retry_time)) {
 	start_retry(nadata);
     } else {
 	nadata->state = CONACCNA_DEAD;
@@ -1117,19 +1114,21 @@ conacc_base_acc_op(struct gensio_accepter *acc, int func,
     }
 }
 
-int
-conacc_gensio_accepter_alloc(const char *gensio_str,
+static int
+conacc_gensio_accepter_alloc(const void *gdata,
 			     const char * const args[],
 			     struct gensio_os_funcs *o,
 			     gensio_accepter_event cb, void *user_data,
 			     struct gensio_accepter **accepter)
 {
+    const char *gensio_str = gdata;
     struct conaccna_data *nadata;
-    unsigned int i, retry_time = 0;
+    unsigned int i;
+    struct gensio_time retry_time = { 0, 0 };
     int err;
 
     for (i = 0; args && args[i]; i++) {
-	if (gensio_check_keyuint(args[i], "retry-time", &retry_time) > 0)
+	if (gensio_check_keytime(args[i], "retry-time", 'm', &retry_time) > 0)
 	    continue;
 	return GE_INVAL;
     }
@@ -1174,7 +1173,7 @@ conacc_gensio_accepter_alloc(const char *gensio_str,
     return err;
 }
 
-int
+static int
 str_to_conacc_gensio_accepter(const char *str, const char * const args[],
 			      struct gensio_os_funcs *o,
 			      gensio_accepter_event cb,
@@ -1183,3 +1182,15 @@ str_to_conacc_gensio_accepter(const char *str, const char * const args[],
 {
     return conacc_gensio_accepter_alloc(str, args, o, cb, user_data, acc);
 }
+
+int
+gensio_init_conacc(struct gensio_os_funcs *o)
+{
+    int rv;
+
+    rv = register_gensio_accepter(o, "conacc", str_to_conacc_gensio_accepter,
+				  conacc_gensio_accepter_alloc);
+    if (rv)
+	return rv;
+    return 0;
+}
