@@ -16,7 +16,6 @@
 #include <gensio/gensio_os_funcs.h>
 #include <gensio/gensio_class.h>
 #include <gensio/argvutils.h>
-#include <gensio/gensio_builtins.h>
 
 #if !USE_FILE_STDIO
 #include <errno.h>
@@ -676,8 +675,9 @@ process_file_args(const char * const args[], struct file_ndata_data *data)
     return 0;
 }
 
-int
-file_gensio_alloc(const char * const args[],
+static int
+file_gensio_alloc(const void *gdata,
+		  const char * const args[],
 		  struct gensio_os_funcs *o,
 		  gensio_event cb, void *user_data,
 		  struct gensio **new_gensio)
@@ -710,11 +710,22 @@ file_gensio_alloc(const char * const args[],
     return GE_NOMEM;
 }
 
-int
+static int
 str_to_file_gensio(const char *str, const char * const args[],
 		   struct gensio_os_funcs *o,
 		   gensio_event cb, void *user_data,
 		   struct gensio **new_gensio)
 {
-    return file_gensio_alloc(args, o, cb, user_data, new_gensio);
+    return file_gensio_alloc(NULL, args, o, cb, user_data, new_gensio);
+}
+
+int
+gensio_init_file(struct gensio_os_funcs *o)
+{
+    int rv;
+
+    rv = register_gensio(o, "file", str_to_file_gensio, file_gensio_alloc);
+    if (rv)
+	return rv;
+    return 0;
 }
