@@ -12,8 +12,6 @@
 
 #include "gensio_filter_ssl.h"
 
-#if HAVE_OPENSSL
-
 #include <assert.h>
 #include <string.h>
 
@@ -30,11 +28,6 @@
 #define DIRSEP '/'
 #endif
 
-/* In gensio_filter_ssl.c, semi-private. */
-int gensio_cert_get_name(X509 *cert, char *data, gensiods *datalen);
-int gensio_cert_to_buf(X509 *cert, char *buf, gensiods *datalen);
-int gensio_cert_fingerprint(X509 *cert, char *buf, gensiods *buflen);
-
 struct gensio_ssl_filter_data {
     struct gensio_os_funcs *o;
     bool is_client;
@@ -86,6 +79,8 @@ struct ssl_filter {
     gensiods read_data_len;
     gensiods max_read_size;
 
+    bool in_ul_handler;
+
     /*
      * This is data from the user waiting to be sent to SSL_write().  This
      * is required because if SSL_write() return that it needs I/O, it must
@@ -232,7 +227,7 @@ ssl_ll_read_needed(struct gensio_filter *filter)
 }
 
 static bool
-ssl_ll_can_write(struct gensio_filter *filter, bool *val)
+ssl_ul_can_write(struct gensio_filter *filter, bool *val)
 {
     struct ssl_filter *sfilter = filter_to_ssl(filter);
 
@@ -644,11 +639,14 @@ ssl_ll_write(struct gensio_filter *filter,
     if (!err && sfilter->read_data_len) {
 	gensiods count = 0;
 
+	assert(!sfilter->in_ul_handler);
+	sfilter->in_ul_handler = true;
 	ssl_unlock(sfilter);
 	err = handler(cb_data, &count,
 		      sfilter->read_data + sfilter->read_data_pos,
 		      sfilter->read_data_len, NULL);
 	ssl_lock(sfilter);
+	sfilter->in_ul_handler = false;
 	if (!err) {
 	    if (count >= sfilter->read_data_len) {
 		sfilter->read_data_len = 0;
@@ -767,7 +765,8 @@ ssl_free(struct gensio_filter *filter)
     return sfilter_free(sfilter);
 }
 
-int
+/* Also in gensio_filter_certauth.c. */
+static int
 gensio_cert_get_name(X509 *cert, char *data, gensiods *datalen)
 {
     char *nidstr = NULL, *end;
@@ -823,7 +822,8 @@ gensio_cert_get_name(X509 *cert, char *data, gensiods *datalen)
     return 0;
 }
 
-int
+/* Also in gensio_filter_certauth.c. */
+static int
 gensio_cert_to_buf(X509 *cert, char *buf, gensiods *buflen)
 {
     BIO *mbio;
@@ -851,7 +851,8 @@ gensio_cert_to_buf(X509 *cert, char *buf, gensiods *buflen)
     return 0;
 }
 
-int
+/* Also in gensio_filter_certauth.c. */
+static int
 gensio_cert_fingerprint(X509 *cert, char *buf, gensiods *buflen)
 {
     gensiods len = *buflen, clen;
@@ -986,8 +987,8 @@ static int gensio_ssl_filter_func(struct gensio_filter *filter, int op,
     case GENSIO_FILTER_FUNC_LL_READ_NEEDED:
 	return ssl_ll_read_needed(filter);
 
-    case GENSIO_FILTER_FUNC_LL_CAN_WRITE:
-	return ssl_ll_can_write(filter, data);
+    case GENSIO_FILTER_FUNC_UL_CAN_WRITE:
+	return ssl_ul_can_write(filter, data);
 
     case GENSIO_FILTER_FUNC_CHECK_OPEN_DONE:
 	return ssl_check_open_done(filter, data);
@@ -1387,27 +1388,3 @@ gensio_ssl_filter_alloc(struct gensio_ssl_filter_data *data,
     SSL_CTX_free(ctx);
     return rv;
 }
-#else /* HAVE_OPENSSL */
-
-int
-gensio_ssl_filter_config(struct gensio_os_funcs *o,
-			 const char * const args[],
-			 bool default_is_client,
-			 struct gensio_ssl_filter_data **rdata)
-{
-    return GE_NOTSUP;
-}
-
-void
-gensio_ssl_filter_config_free(struct gensio_ssl_filter_data *data)
-{
-}
-
-int
-gensio_ssl_filter_alloc(struct gensio_ssl_filter_data *data,
-			struct gensio_filter **rfilter)
-{
-    return GE_NOTSUP;
-}
-
-#endif /* HAVE_OPENSSL */
