@@ -6,7 +6,9 @@
  */
 
 #include "config.h"
+#include <gensio/gensio.h>
 #include <gensio/gensio_class.h>
+#include <gensio/gensio_os_funcs.h>
 #include <gensio/gensio_ll_gensio.h>
 
 #include <assert.h>
