@@ -11,10 +11,8 @@
  */
 
 #include "config.h"
-#include <gensio/gensio_builtins.h>
 #include <gensio/gensio_err.h>
 
-#ifdef HAVE_AVAHI
 #include <assert.h>
 #include <string.h>
 #include <stdio.h>
@@ -462,13 +460,15 @@ mdns_cb(struct gensio_mdns_watch *w,
 		goto out_err;
 
 	    if (strcmp(s, "_tcp") == 0) {
-		ndata->open_err = tcp_gensio_alloc(addr, argv, ndata->o,
-						   child_cb, ndata,
-						   &ndata->child);
+		ndata->open_err = gensio_terminal_alloc("tcp",
+						addr, argv, ndata->o,
+						child_cb, ndata,
+						&ndata->child);
 	    } else if (strcmp(s, "_udp") == 0) {
-		ndata->open_err = udp_gensio_alloc(addr, argv, ndata->o,
-						   child_cb, ndata,
-						   &ndata->child);
+		ndata->open_err = gensio_terminal_alloc("udp",
+						addr, argv, ndata->o,
+						child_cb, ndata,
+						&ndata->child);
 	    } else {
 		goto out_unlock;
 	    }
@@ -716,12 +716,13 @@ mdns_ndata_setup(struct gensio_os_funcs *o, gensiods max_read_size,
     return GE_NOMEM;
 }
 
-int
-mdns_gensio_alloc(const char *mstr, const char * const args[],
+static int
+mdns_gensio_alloc(const void *gdata, const char * const args[],
 		  struct gensio_os_funcs *o,
 		  gensio_event cb, void *user_data,
 		  struct gensio **new_gensio)
 {
+    const char *mstr = gdata;
     int err;
     struct mdnsn_data *ndata = NULL;
     int i, interface = -1, nettype = GENSIO_NETTYPE_UNSPEC;
@@ -916,7 +917,7 @@ mdns_gensio_alloc(const char *mstr, const char * const args[],
     return err;
 }
 
-int
+static int
 str_to_mdns_gensio(const char *str, const char * const args[],
 		   struct gensio_os_funcs *o,
 		   gensio_event cb, void *user_data,
@@ -925,24 +926,13 @@ str_to_mdns_gensio(const char *str, const char * const args[],
     return mdns_gensio_alloc(str, args, o, cb, user_data, new_gensio);
 }
 
-#else
-
 int
-mdns_gensio_alloc(const char *str, const char * const args[],
-		  struct gensio_os_funcs *o,
-		  gensio_event cb, void *user_data,
-		  struct gensio **new_gensio)
+gensio_init_mdns(struct gensio_os_funcs *o)
 {
-    return GE_NOTSUP;
-}
+    int rv;
 
-int
-str_to_mdns_gensio(const char *str, const char * const args[],
-		   struct gensio_os_funcs *o,
-		   gensio_event cb, void *user_data,
-		   struct gensio **new_gensio)
-{
-    return GE_NOTSUP;
+    rv = register_gensio(o, "mdns", str_to_mdns_gensio, mdns_gensio_alloc);
+    if (rv)
+	return rv;
+    return 0;
 }
-
-#endif
