@@ -13,11 +13,10 @@
 #include <gensio/gensio_ll_gensio.h>
 #include <gensio/gensio_acc_gensio.h>
 #include <gensio/argvutils.h>
-#include <gensio/gensio_builtins.h>
 
 #include "gensio_filter_msgdelim.h"
 
-int
+static int
 msgdelim_gensio_alloc(struct gensio *child, const char *const args[],
 		      struct gensio_os_funcs *o,
 		      gensio_event cb, void *user_data,
@@ -53,7 +52,7 @@ msgdelim_gensio_alloc(struct gensio *child, const char *const args[],
     return 0;
 }
 
-int
+static int
 str_to_msgdelim_gensio(const char *str, const char * const args[],
 		       struct gensio_os_funcs *o,
 		       gensio_event cb, void *user_data,
@@ -137,7 +136,7 @@ gensio_gensio_acc_msgdelim_cb(void *acc_data, int op, void *data1, void *data2,
     }
 }
 
-int
+static int
 msgdelim_gensio_accepter_alloc(struct gensio_accepter *child,
 			       const char * const args[],
 			       struct gensio_os_funcs *o,
@@ -174,7 +173,7 @@ msgdelim_gensio_accepter_alloc(struct gensio_accepter *child,
     return err;
 }
 
-int
+static int
 str_to_msgdelim_gensio_accepter(const char *str, const char * const args[],
 				struct gensio_os_funcs *o,
 				gensio_accepter_event cb,
@@ -193,3 +192,20 @@ str_to_msgdelim_gensio_accepter(const char *str, const char * const args[],
 
     return err;
 }
+
+int
+gensio_init_msgdelim(struct gensio_os_funcs *o)
+{
+    int rv;
+
+    rv = register_filter_gensio(o, "msgdelim",
+				str_to_msgdelim_gensio, msgdelim_gensio_alloc);
+    if (rv)
+	return rv;
+    rv = register_filter_gensio_accepter(o, "msgdelim",
+					 str_to_msgdelim_gensio_accepter,
+					 msgdelim_gensio_accepter_alloc);
+    if (rv)
+	return rv;
+    return 0;
+}
