@@ -15,7 +15,6 @@
 #include <gensio/gensio_os_funcs.h>
 #include <gensio/gensio_class.h>
 #include <gensio/gensio_acc_gensio.h>
-#include <gensio/gensio_builtins.h>
 #include <gensio/gensio_list.h>
 
 /*
@@ -703,14 +702,14 @@ static bool i_chan_deref(struct mux_inst *chan)
 #ifdef MUX_TRACING
 #define chan_ref(c) do {						\
 	i_mux_add_trace(c->mux, c, c->mux->state, 106,			\
-			c->state, c->refcount + 2000, __LINE__);	\
+			c->state, c->refcount + 3000, __LINE__);	\
 	i_chan_ref(c);							\
     } while (false)
 
 static bool i2_chan_deref(struct mux_inst *chan, int line)
 {
     i_mux_add_trace(chan->mux, chan, chan->mux->state, 107,
-		    chan->state, chan->refcount + 2000, line);
+		    chan->state, chan->refcount + 3000, line);
     return i_chan_deref(chan);
 }
 
@@ -970,8 +969,11 @@ chan_check_read(struct mux_inst *chan)
 	chan->in_read_report = true;
 	i = 0;
 	if (flags & MUX_FLAG_OUT_OF_BOUND) {
-	    if (!chan->do_oob)
-		goto after_read_done;
+	    if (!chan->do_oob) {
+		rcount = len;
+		orcount = rcount;
+		goto skip_oob_data;
+	    }
 	    flstr[i++] = "oob";
 	}
 	flstr[i] = NULL;
@@ -989,6 +991,7 @@ chan_check_read(struct mux_inst *chan)
 	    }
 	    if (rcount > orcount)
 		rcount = orcount;
+	skip_oob_data:
 	    len -= rcount;
 	    to_ack += rcount;
 	    olen += rcount;
@@ -2313,8 +2316,10 @@ mux_child_read(struct mux_data *muxdata, int ierr,
 	    switch (muxdata->msgid) {
 	    case MUX_NEW_CHANNEL: {
 		unsigned int remote_id = gensio_buf_to_u16(muxdata->hdr + 2);
+		bool was_chan0 = false;
 		if (muxdata->state == MUX_WAITING_OPEN) {
 		    chan = mux_chan0(muxdata);
+		    was_chan0 = true;
 		} else {
 		    int err;
 
@@ -2336,7 +2341,10 @@ mux_child_read(struct mux_data *muxdata, int ierr,
 			gensio_buf_to_u32(muxdata->hdr + 4);
 		    if (chan->send_window_size <= MUX_MIN_SEND_WINDOW_SIZE) {
 			proto_err_str = "Invalid send window size";
-			goto protocol_err;
+			if (was_chan0)
+			    goto protocol_err;
+			else
+			    goto protocol_err_close_chan;
 		    }
 		    chan->remote_id = remote_id;
 		    muxdata->data_pos = 0;
@@ -2553,7 +2561,7 @@ mux_child_read(struct mux_data *muxdata, int ierr,
 		    if (muxdata->xmit_data_len) {
 			/* Only one new channel allowed at a time. */
 			proto_err_str = "New channel while in progress";
-			goto protocol_err;
+			goto protocol_err_close_chan;
 		    }
 		    mux_send_new_channel_rsp(muxdata,
 				gensio_buf_to_u16(muxdata->hdr + 2),
@@ -2582,7 +2590,7 @@ mux_child_read(struct mux_data *muxdata, int ierr,
 		} else {
 		    if (muxdata->xmit_data_len) {
 			proto_err_str = "New channel while in progress";
-			goto protocol_err;
+			goto protocol_err_close_chan;
 		    }
 		    if (chan->service)
 			auxdata[0] = chan->service;
@@ -2664,6 +2672,15 @@ mux_child_read(struct mux_data *muxdata, int ierr,
     *ibuflen = processed;
     return 0;
 
+ protocol_err_close_chan:
+    /*
+     * A protocol error was reported before the channel was reported
+     * to the user.  Just delete the channel, otherwise it will not
+     * get cleaned up on the close and the close of the mux will never
+     * happen.
+     */
+    chan_deref(muxdata->curr_chan);
+    muxdata->curr_chan = NULL;
  protocol_err:
     gmux_log_err(muxdata, "Protocol error: %s\n", proto_err_str);
     ierr = GE_PROTOERR;
@@ -2764,7 +2781,7 @@ mux_gensio_alloc_data(struct gensio *child, struct gensio_mux_config *data,
     return GE_NOMEM;
 }
 
-int
+static int
 mux_gensio_alloc(struct gensio *child, const char *const args[],
 		 struct gensio_os_funcs *o,
 		 gensio_event cb, void *user_data,
@@ -2813,7 +2830,7 @@ mux_gensio_alloc(struct gensio *child, const char *const args[],
     return 0;
 }
 
-int
+static int
 str_to_mux_gensio(const char *str, const char * const args[],
 		  struct gensio_os_funcs *o,
 		  gensio_event cb, void *user_data,
@@ -2907,7 +2924,7 @@ gensio_gensio_acc_mux_cb(void *acc_data, int op, void *data1, void *data2,
     }
 }
 
-int
+static int
 mux_gensio_accepter_alloc(struct gensio_accepter *child,
 			  const char * const args[],
 			  struct gensio_os_funcs *o,
@@ -2965,7 +2982,7 @@ mux_gensio_accepter_alloc(struct gensio_accepter *child,
     return err;
 }
 
-int
+static int
 str_to_mux_gensio_accepter(const char *str, const char * const args[],
 			   struct gensio_os_funcs *o,
 			   gensio_accepter_event cb,
@@ -2984,3 +3001,20 @@ str_to_mux_gensio_accepter(const char *str, const char * const args[],
 
     return err;
 }
+
+int
+gensio_init_mux(struct gensio_os_funcs *o)
+{
+    int rv;
+
+    rv = register_filter_gensio(o, "mux",
+				str_to_mux_gensio, mux_gensio_alloc);
+    if (rv)
+	return rv;
+    rv = register_filter_gensio_accepter(o, "mux",
+					 str_to_mux_gensio_accepter,
+					 mux_gensio_accepter_alloc);
+    if (rv)
+	return rv;
+    return 0;
+}
