@@ -31,7 +31,6 @@
 #include <gensio/gensio_ll_fd.h>
 #include <gensio/argvutils.h>
 #include <gensio/gensio_osops.h>
-#include <gensio/gensio_builtins.h>
 
 #include "gensio_net.h"
 
@@ -473,16 +472,18 @@ str_to_net_gensio(const char *str, const char * const args[],
     return err;
 }
 
-int
-tcp_gensio_alloc(const struct gensio_addr *iai, const char * const args[],
+static int
+tcp_gensio_alloc(const void *gdata, const char * const args[],
 		 struct gensio_os_funcs *o,
 		 gensio_event cb, void *user_data,
 		 struct gensio **new_gensio)
 {
+    const struct gensio_addr *iai = gdata;
+
     return net_gensio_alloc(iai, args, o, cb, user_data, "tcp", new_gensio);
 }
 
-int
+static int
 str_to_tcp_gensio(const char *str, const char * const args[],
 		  struct gensio_os_funcs *o,
 		  gensio_event cb, void *user_data,
@@ -492,20 +493,22 @@ str_to_tcp_gensio(const char *str, const char * const args[],
 			     o, cb, user_data, new_gensio);
 }
 
-int
-unix_gensio_alloc(const struct gensio_addr *iai, const char * const args[],
+static int
+unix_gensio_alloc(const void *gdata, const char * const args[],
 		 struct gensio_os_funcs *o,
 		 gensio_event cb, void *user_data,
 		 struct gensio **new_gensio)
 {
 #if HAVE_UNIX
+    const struct gensio_addr *iai = gdata;
+
     return net_gensio_alloc(iai, args, o, cb, user_data, "unix", new_gensio);
 #else
     return GE_NOTSUP;
 #endif
 }
 
-int
+static int
 str_to_unix_gensio(const char *str, const char * const args[],
 		   struct gensio_os_funcs *o,
 		   gensio_event cb, void *user_data,
@@ -1120,7 +1123,7 @@ netna_base_acc_op(struct gensio_accepter *acc, int op,
 }
 
 #ifdef HAVE_TCPD_H
-struct gensio_enum_val tcpd_enums[] = {
+static struct gensio_enum_val tcpd_enums[] = {
     { "on",	GENSIO_TCPD_ON },
     { "print",	GENSIO_TCPD_PRINT },
     { "off", 	GENSIO_TCPD_OFF },
@@ -1129,7 +1132,7 @@ struct gensio_enum_val tcpd_enums[] = {
 #endif
 
 static int
-net_gensio_accepter_alloc(struct gensio_addr *iai,
+net_gensio_accepter_alloc(const struct gensio_addr *iai,
 			  const char * const args[],
 			  struct gensio_os_funcs *o,
 			  gensio_accepter_event cb, void *user_data,
@@ -1303,18 +1306,20 @@ str_to_net_gensio_accepter(const char *str, const char * const args[],
     return err;
 }
 
-int
-tcp_gensio_accepter_alloc(struct gensio_addr *iai,
+static int
+tcp_gensio_accepter_alloc(const void *gdata,
 			  const char * const args[],
 			  struct gensio_os_funcs *o,
 			  gensio_accepter_event cb, void *user_data,
 			  struct gensio_accepter **accepter)
 {
+    const struct gensio_addr *iai = gdata;
+
     return net_gensio_accepter_alloc(iai, args, o, cb, user_data, "tcp",
 				     accepter);
 }
 
-int
+static int
 str_to_tcp_gensio_accepter(const char *str, const char * const args[],
 			   struct gensio_os_funcs *o,
 			   gensio_accepter_event cb,
@@ -1325,14 +1330,16 @@ str_to_tcp_gensio_accepter(const char *str, const char * const args[],
 				      o, cb, user_data, acc);
 }
 
-int
-unix_gensio_accepter_alloc(struct gensio_addr *iai,
+static int
+unix_gensio_accepter_alloc(const void *gdata,
 			   const char * const args[],
 			   struct gensio_os_funcs *o,
 			   gensio_accepter_event cb, void *user_data,
 			   struct gensio_accepter **accepter)
 {
 #if HAVE_UNIX
+    const struct gensio_addr *iai = gdata;
+
     return net_gensio_accepter_alloc(iai, args, o, cb, user_data, "unix",
 				     accepter);
 #else
@@ -1340,7 +1347,7 @@ unix_gensio_accepter_alloc(struct gensio_addr *iai,
 #endif
 }
 
-int
+static int
 str_to_unix_gensio_accepter(const char *str, const char * const args[],
 			    struct gensio_os_funcs *o,
 			    gensio_accepter_event cb,
@@ -1350,3 +1357,25 @@ str_to_unix_gensio_accepter(const char *str, const char * const args[],
     return str_to_net_gensio_accepter(str, args, GENSIO_NET_PROTOCOL_UNIX,
 				      "unix", o, cb, user_data, acc);
 }
+
+int
+gensio_init_net(struct gensio_os_funcs *o)
+{
+    int rv;
+
+    rv = register_gensio(o, "tcp", str_to_tcp_gensio, tcp_gensio_alloc);
+    if (rv)
+	return rv;
+    rv = register_gensio_accepter(o, "tcp", str_to_tcp_gensio_accepter,
+				  tcp_gensio_accepter_alloc);
+    if (rv)
+	return rv;
+    rv = register_gensio(o, "unix", str_to_unix_gensio, unix_gensio_alloc);
+    if (rv)
+	return rv;
+    rv = register_gensio_accepter(o, "unix", str_to_unix_gensio_accepter,
+				  unix_gensio_accepter_alloc);
+    if (rv)
+	return rv;
+    return 0;
+}
