@@ -22,6 +22,7 @@
 
 #include <gensio/gensio.h>
 #include <gensio/gensio_os_funcs.h>
+#include <gensio/gensio_os_funcs_public.h>
 #include <gensio/gensio_osops.h>
 #include <gensio/gensio_class.h>
 #include <gensio/argvutils.h>
@@ -37,6 +38,11 @@ bool gensio_set_progname(const char *iprogname)
     return true;
 }
 
+const char *gensio_get_progname(void)
+{
+    return progname;
+}
+
 
 #ifdef _WIN32
 #include <winsock2.h> /* For AF_UNSPEC */
@@ -53,44 +59,57 @@ bool gensio_set_progname(const char *iprogname)
 #include <ifaddrs.h>
 #include <netinet/in.h>
 #include <net/if.h>
+#include <limits.h>
+#include <dlfcn.h>
 
 int
 gensio_unix_os_setupnewprog(void)
 {
     struct passwd *pw;
     int err;
-    uid_t uid = geteuid();
+    uid_t uid = getuid();
     gid_t *groups = NULL;
     int ngroup = 0;
 
     if (do_errtrig())
 	return GE_NOMEM;
 
-    if (uid == getuid())
+    if (uid == geteuid())
 	return 0;
 
-    err = seteuid(getuid());
-    if (err)
-	return errno;
-
     pw = getpwuid(uid);
     if (!pw)
 	return errno;
 
+    /* Sets the real, effective, and saved group. */
+    err = setgid(getgid());
+    if (err)
+	return errno;
+
     getgrouplist(pw->pw_name, pw->pw_gid, groups, &ngroup);
     if (ngroup > 0) {
+	int ngroup2 = ngroup;
+
 	groups = malloc(sizeof(gid_t) * ngroup);
 	if (!groups)
 	    return ENOMEM;
 
-	err = getgrouplist(pw->pw_name, pw->pw_gid, groups, &ngroup);
+	err = getgrouplist(pw->pw_name, pw->pw_gid, groups, &ngroup2);
 	if (err == -1) {
 	    err = errno;
 	    free(groups);
 	    return err;
 	}
+	/*
+	 * There is some possibility that the number of groups will
+	 * change between the calls.  If the new on is bigger than the
+	 * old, use the old one to avoid accessing past the end of the
+	 * buffer.  If the new one is smaller, use that.
+	 */
+	if (ngroup2 < ngroup)
+	    ngroup = ngroup2;
 
-	err = setgroups(err, groups);
+	err = setgroups(ngroup, groups);
 	if (err) {
 	    err = errno;
 	    free(groups);
@@ -99,13 +118,11 @@ gensio_unix_os_setupnewprog(void)
 	free(groups);
     }
 
-    err = setgid(getegid());
-    if (err)
-	return errno;
-
+    /* Sets the real, effective, and saved userid. */
     err = setuid(uid);
     if (err)
 	return errno;
+
     return 0;
 }
 #endif
@@ -1160,7 +1177,8 @@ argv_to_win_cmdline(struct gensio_os_funcs *o, const char *argv[],
  */
 static char *
 win_env_to_block(struct gensio_os_funcs *o,
-		 const char **env)
+		 const char **env,
+		 gensiods *rsize)
 {
     /*
      * Start with size=2 because this must be terminated with two nil
@@ -1181,6 +1199,8 @@ win_env_to_block(struct gensio_os_funcs *o,
 	memcpy(pos, env[i], len);
 	pos += len + 1;
     }
+    if (rsize)
+	*rsize = size;
     return envb;
 }
 
@@ -1209,7 +1229,7 @@ gensio_win_do_exec(struct gensio_os_funcs *o,
 	return rv;
 
     if (env) {
-	envb = win_env_to_block(o, env);
+	envb = win_env_to_block(o, env, NULL);
 	if (!envb) {
 	    rv = GE_NOMEM;
 	    goto out;
@@ -1254,7 +1274,8 @@ gensio_win_do_exec(struct gensio_os_funcs *o,
 			     0, TRUE, DUPLICATE_SAME_ACCESS))
 	    goto out_err_conv;
     }
-    if (!SetHandleInformation(stderr_s, HANDLE_FLAG_INHERIT, 1))
+    if (!SetHandleInformation(stderr_s, HANDLE_FLAG_INHERIT,
+			      HANDLE_FLAG_INHERIT))
 	goto out_err_conv;
 
     suinfo.cb = sizeof(STARTUPINFO);
@@ -1690,6 +1711,24 @@ print_sid(const char *str, SID *sid)
 }
 #endif
 
+static wchar_t *
+str_to_wstr(const char *str)
+{
+    DWORD nchars;
+    DWORD nbytes;
+    wchar_t *wstr;
+
+    if (!str)
+	return NULL;
+    nchars = mbstowcs(NULL, str, strlen(str));
+    nbytes = nchars * sizeof(wchar_t);
+    wstr = calloc(nbytes + sizeof(wchar_t), 1);
+    if (!wstr)
+	return NULL;
+    mbstowcs(wstr, str, nchars);
+    return wstr;
+}
+
 int
 gensio_win_get_user_token(const char *user, const char *password,
 			  const char *src_module, const char **groups,
@@ -1701,8 +1740,6 @@ gensio_win_get_user_token(const char *user, const char *password,
     void *logon_info = NULL;
     LSA_STRING package_name;
     ULONG package_auth;
-    DWORD user_chars = mbstowcs(NULL, user, strlen(user));
-    DWORD user_bytes = user_chars * sizeof(wchar_t);
     wchar_t *wuser = NULL;
     bool domain_user;
     DWORD logon_len;
@@ -1875,12 +1912,11 @@ gensio_win_get_user_token(const char *user, const char *password,
 
     profile_info.dwSize = sizeof(profile_info);
     profile_info.dwFlags = PI_NOUI;
-    wuser = calloc(user_bytes + sizeof(wchar_t), 1);
+    wuser = str_to_wstr(user);
     if (!wuser) {
 	err = STATUS_NO_MEMORY;
 	goto out_err;
     }
-    mbstowcs(wuser, user, user_chars);
     profile_info.lpUserName = wuser;
     if (!LoadUserProfileW(htok, &profile_info)) {
 	err = GetLastError();
@@ -1932,39 +1968,41 @@ gensio_win_pty_alloc(struct gensio_os_funcs *o,
 {
     HANDLE readh_m = NULL, readh_s = NULL;
     HANDLE writeh_m = NULL, writeh_s = NULL;
+    HANDLE imptokh;
     HPCON ptyh = NULL;
     COORD winsize;
     HRESULT hr;
-    HANDLE imptokh = NULL;
     int err;
 
-    /*
-     * We can't create pipes or the pseudoconsole with the default
-     * token set because it can't access the default security token.
-     * Briefly go back to the main access token to do this.
-     */
+    if (!CreatePipe(&writeh_s, &writeh_m, NULL, 0))
+	goto out_err_conv;
+    if (!SetHandleInformation(writeh_s, HANDLE_FLAG_INHERIT,
+			      HANDLE_FLAG_INHERIT))
+	goto out_err_conv;
+    if (!CreatePipe(&readh_m, &readh_s, NULL, 0))
+	goto out_err_conv;
+    if (!SetHandleInformation(readh_s, HANDLE_FLAG_INHERIT,
+			      HANDLE_FLAG_INHERIT))
+	goto out_err_conv;
+
     if (!OpenThreadToken(GetCurrentThread(),
 			 TOKEN_ALL_ACCESS,
 			 TRUE,
 			 &imptokh)) {
 	if (GetLastError() != ERROR_NO_TOKEN)
+	    /* Error getting the token. */
 	    goto out_err_conv;
+	/* No token was present, just create the pseudo console. */
     } else {
-	if (!RevertToSelf()) {
-	    CloseHandle(imptokh);
-	    imptokh = NULL;
-	    goto out_err_conv;
-	}
+	/*
+	 * An impersonation token is present, the pseudo console will
+	 * be created in a special child process.
+	 */
+	CloseHandle(imptokh);
+	goto out;
     }
 
-    if (!CreatePipe(&writeh_s, &writeh_m, NULL, 0))
-	goto out_err_conv;
-    if (!SetHandleInformation(writeh_s, HANDLE_FLAG_INHERIT, 0))
-	goto out_err_conv;
-    if (!CreatePipe(&readh_m, &readh_s, NULL, 0))
-	goto out_err_conv;
-    if (!SetHandleInformation(readh_s, HANDLE_FLAG_INHERIT, 0))
-	goto out_err_conv;
+
     winsize.X = 80;
     winsize.Y = 25;
     hr = CreatePseudoConsole(winsize, writeh_s, readh_s, 0, &ptyh);
@@ -1976,12 +2014,7 @@ gensio_win_pty_alloc(struct gensio_os_funcs *o,
 	goto out_err;
     }
 
-    /* Go back to the impersonation token. */
-    if (imptokh) {
-	if (!SetThreadToken(NULL, imptokh))
-	    goto out_err_conv;
-    }
-
+ out:
     *child_in = writeh_s;
     *child_out = readh_s;
     *rwriteh = writeh_m;
@@ -2006,17 +2039,132 @@ gensio_win_pty_alloc(struct gensio_os_funcs *o,
     return err;
 }
 
+/*
+ * Start a special process that will be an intermediary between this
+ * process and the main user process.
+ *
+ * If you create a pseudo console under Windows, it will be created
+ * with the integrity level of the calling process, even if the
+ * impersonation token is set.  This means that the child process will
+ * not be able to do certain operations on the console handles, like
+ * WriteConsoleInput.  A discussion of the security limitiations is at
+ * https://learn.microsoft.com/en-us/windows/console/console-buffer-security-and-access-rights
+ *
+ * This was discussed by me with others at:
+ *
+ * https://learn.microsoft.com/en-us/answers/questions/1040676/createpseudoconsole-with-reduced-integrity-level.html
+ *
+ * The only reasonable approach seems to be to create another process
+ * with the user's token and have it create the pty.  That process is
+ * gensio_pty_helper in tools.  That way the pty gets created at the
+ * user's integrity level.  We talk to that process over it's stdio,
+ * which is relayed to the real child process.
+ *
+ * The environment and starting directory can be set here and it will
+ * propagate down.  However, the command line is a different issue.
+ * That is passed in some shared memory.
+ *
+ * There is yet another snag, though.  The console resize events need
+ * to be done on the pty.  So another pipe is created to send commands
+ * to gensio_pty_helper to handle resize events.
+ *
+ * The command line shared memory handle, size, and the rcontrol
+ * shared memory handle is passed on the command line to
+ * gensio_pty_helper.
+ *
+ * Also, the exit code must be propagated back through gensio_pty_helper.
+ *
+ * This is not really ideal, as the subprocess is not directly
+ * attached to us and the pid really isn't right.  But there's not
+ * much else we can do.
+ */
+static int
+start_pty_helper(struct gensio_os_funcs *o,
+		 HANDLE tokh, const char *cmdline, char *envb,
+		 const char *start_dir, STARTUPINFO *si,
+		 PROCESS_INFORMATION *procinfo, HANDLE *rcontrol)
+{
+    SECURITY_ATTRIBUTES secattr;
+    HANDLE shmem = NULL, ctl_s = NULL, ctl_m = NULL;
+    DWORD cmdline_len;
+    char *buf = NULL;
+    char cmd[50];
+    int err = 0;
+
+    if (!CreatePipe(&ctl_s, &ctl_m, NULL, 0))
+	goto out_err_conv;
+    if (!SetHandleInformation(ctl_s, HANDLE_FLAG_INHERIT, HANDLE_FLAG_INHERIT))
+	goto out_err_conv;
+
+    cmdline_len = strlen(cmdline) + 1;
+
+    memset(&secattr, 0, sizeof(secattr));
+    secattr.nLength = sizeof(secattr);
+    secattr.bInheritHandle = TRUE;
+
+    /* Add 1 to the length for the version. */
+    shmem = CreateFileMappingA(INVALID_HANDLE_VALUE, &secattr, PAGE_READWRITE,
+			       0, cmdline_len + 1, NULL);
+    if (!shmem)
+	goto out_err_conv;
+
+    buf = (char *) MapViewOfFile(shmem,
+				 FILE_MAP_ALL_ACCESS,
+				 0,
+				 0,
+				 cmdline_len + 1);
+    if (!buf)
+	goto out_err_conv;
+    buf[0] = 1; /* Version */
+    memcpy(buf + 1, cmdline, cmdline_len);
+    UnmapViewOfFile(buf);
+
+    /* Pass the handle in the command line. */
+    snprintf(cmd, sizeof(cmd), "gensio_pty_helper %lld %lu %lld",
+	     (intptr_t) shmem, (unsigned long) cmdline_len + 1,
+	     (intptr_t) ctl_s);
+
+    if (!CreateProcessAsUserA(tokh,
+			      NULL,
+			      cmd,
+			      NULL,
+			      NULL,
+			      TRUE,
+			      (NORMAL_PRIORITY_CLASS |
+			       CREATE_NEW_PROCESS_GROUP),
+			      envb,
+			      start_dir,
+			      si,
+			      procinfo))
+	goto out_err_conv;
+
+ out:
+    if (shmem)
+	CloseHandle(shmem);
+    if (ctl_s)
+	CloseHandle(ctl_s);
+    if (err && ctl_m)
+	CloseHandle(ctl_m);
+    else
+	*rcontrol = ctl_m;
+    return err;
+
+ out_err_conv:
+    err = gensio_os_err_to_err(o, GetLastError());
+    goto out;
+}
+
 int
 gensio_win_pty_start(struct gensio_os_funcs *o,
 		     HPCON ptyh, HANDLE *child_in, HANDLE *child_out,
 		     const char **argv, const char **env,
-		     const char *start_dir, HANDLE *child)
+		     const char *start_dir, HANDLE *child, HANDLE *rcontrol)
 {
     char *cmdline, *envb = NULL;
     STARTUPINFOEX si;
     PROCESS_INFORMATION procinfo;
     size_t len;
-    HANDLE tokh = NULL, imptokh = NULL;
+    HANDLE tokh = NULL, imptokh = NULL, control = NULL;
     int err;
     bool setuser = true;
     bool attrs_added = false;
@@ -2028,7 +2176,7 @@ gensio_win_pty_start(struct gensio_os_funcs *o,
 	return err;
 
     if (env) {
-	envb = win_env_to_block(o, env);
+	envb = win_env_to_block(o, env, NULL);
 	if (!envb) {
 	    err = GE_NOMEM;
 	    goto out_err;
@@ -2044,29 +2192,14 @@ gensio_win_pty_start(struct gensio_os_funcs *o,
      * the console, all is good, but that's not the normal case.
      *
      * So set the stdio to the pipe, too.
+     *
+     * Plus this is necessary if we are using the pty helper.
      */
     si.StartupInfo.hStdInput = *child_in;
     si.StartupInfo.hStdOutput = *child_out;
     si.StartupInfo.hStdError = *child_out;
     si.StartupInfo.dwFlags |= STARTF_USESTDHANDLES;
 
-    InitializeProcThreadAttributeList(NULL, 1, 0, &len);
-    si.lpAttributeList = o->zalloc(o, len);
-    if (!si.lpAttributeList) {
-	err = GE_NOMEM;
-	goto out_err;
-    }
-    if (!InitializeProcThreadAttributeList(si.lpAttributeList, 1, 0, &len))
-	goto out_err_conv;
-    if (!UpdateProcThreadAttribute(si.lpAttributeList,
-                                   0,
-                                   PROC_THREAD_ATTRIBUTE_PSEUDOCONSOLE,
-                                   ptyh,
-                                   sizeof(ptyh),
-                                   NULL,
-                                   NULL))
-	goto out_err_conv;
-    attrs_added = true;
 
     /*
      * Get the impersonation token, convert it to a real token, and
@@ -2088,21 +2221,29 @@ gensio_win_pty_start(struct gensio_os_funcs *o,
 			      TokenPrimary, &tokh))
 	    goto out_err_conv;
 
-	if (!CreateProcessAsUserA(tokh,
-				  NULL,
-				  cmdline,
-				  NULL,
-				  NULL,
-				  FALSE,
-				  (NORMAL_PRIORITY_CLASS |
-				   EXTENDED_STARTUPINFO_PRESENT |
-				   CREATE_NEW_PROCESS_GROUP),
-				  envb,
-				  start_dir,
-				  &si.StartupInfo,
-				  &procinfo))
-	    goto out_err_conv;
+	err = start_pty_helper(o, tokh, cmdline, envb, start_dir,
+			       &si.StartupInfo, &procinfo, &control);
+	if (err)
+	    goto out_err;
     } else {
+	InitializeProcThreadAttributeList(NULL, 1, 0, &len);
+	si.lpAttributeList = o->zalloc(o, len);
+	if (!si.lpAttributeList) {
+	    err = GE_NOMEM;
+	    goto out_err;
+	}
+	if (!InitializeProcThreadAttributeList(si.lpAttributeList, 1, 0, &len))
+	    goto out_err_conv;
+	if (!UpdateProcThreadAttribute(si.lpAttributeList,
+				       0,
+				       PROC_THREAD_ATTRIBUTE_PSEUDOCONSOLE,
+				       ptyh,
+				       sizeof(ptyh),
+				       NULL,
+				       NULL))
+	    goto out_err_conv;
+	attrs_added = true;
+
 	if (!CreateProcess(NULL,
 			   cmdline,
 			   NULL,
@@ -2130,6 +2271,7 @@ gensio_win_pty_start(struct gensio_os_funcs *o,
     DeleteProcThreadAttributeList(si.lpAttributeList);
     o->free(o, si.lpAttributeList);
     *child = procinfo.hProcess;
+    *rcontrol = control;
     return 0;
 
  out_err_conv:
@@ -2150,6 +2292,44 @@ gensio_win_pty_start(struct gensio_os_funcs *o,
     return err;
 }
 
+bool
+gensio_os_loadlib(struct gensio_os_funcs *o, const char *name)
+{
+    char *dllname;
+    char *initfuncname;
+    int (*initfunc)(struct gensio_os_funcs *);
+    HINSTANCE handle;
+    int err;
+
+    dllname = gensio_alloc_sprintf(o, "libgensio_%s.DLL", name);
+    if (!dllname)
+	return false;
+    initfuncname = gensio_alloc_sprintf(o, "gensio_init_%s", name);
+    if (!initfuncname) {
+	o->free(o, dllname);
+	return false;
+    }
+    handle = LoadLibrary(dllname);
+    o->free(o, dllname);
+    if (!handle) {
+	o->free(o, initfuncname);
+	return false;
+    }
+    initfunc = (int (*)(struct gensio_os_funcs *))
+	GetProcAddress(handle, initfuncname);
+    o->free(o, initfuncname);
+    if (!initfunc) {
+	FreeLibrary(handle);
+	return false;
+    }
+    err = initfunc(o);
+    if (err) {
+	FreeLibrary(handle);
+	return false;
+    }
+    return true;
+}
+
 #else /* _WIN32 */
 
 #include <fcntl.h>
@@ -3097,11 +3277,8 @@ gensio_unix_pty_start(struct gensio_os_funcs *o,
 	int fd;
 
 	/* Set the owner of the slave PT. */
-	/* FIXME - This should not be necessary, can we remove? */
-#if 0
 	if (grantpt(pfd) < 0)
 	    exit(1);
-#endif
 
 	if (start_dir) {
 	    if (chdir(start_dir)) {
@@ -3117,14 +3294,6 @@ gensio_unix_pty_start(struct gensio_os_funcs *o,
 	    exit(1);
 	}
 
-#if 0 /* FIXME = do we need this? */
-	if (setpgid(0, 0) == -1) {
-	    fprintf(stderr, "pty fork: failed setpgid: %s\r\n",
-		    strerror(errno));
-	    exit(1);
-	}
-#endif
-
 	fd = open(slave, O_RDWR);
 	if (fd == -1) {
 	    fprintf(stderr, "pty fork: failed to open slave terminal: %s\r\n",
@@ -3192,6 +3361,113 @@ gensio_unix_pty_start(struct gensio_os_funcs *o,
     return gensio_os_err_to_err(o, errno);
 }
 
+static bool
+pathendswith(const char *str, const char *endstr)
+{
+    const char *end1 = str + strlen(str) - 1;
+    const char *end2 = endstr + strlen(endstr) - 1;
+
+    /* Ignore trailing '/' */
+    while (end1 != str && *end1 == '/')
+	end1--;
+    if (*end1 == '/')
+	return false; /* It was all /'s */
+
+    while (end1 != str && end2 != endstr && *end1 == *end2) {
+	end1--;
+	end2--;
+    }
+    return end2 == endstr && *end1 == *end2;
+}
+
+static bool
+try_loaddir(struct gensio_os_funcs *o, const char *name, const char *path,
+	    bool check_libtool_dir)
+{
+    gensiods len;
+    char fname[PATH_MAX];
+    void *handle;
+    int (*initfunc)(struct gensio_os_funcs *);
+    int err;
+
+    if (check_libtool_dir) {
+	/*
+	 * Only allow libtool libraries, so when running from in the
+	 * build directory it will find it.)
+	 */
+	if (!pathendswith(path, "/lib/.libs"))
+	    return false;
+    }
+    len = snprintf(fname, sizeof(fname), "%s/libgensio_%s.so", path, name);
+    if (len >= sizeof(fname))
+	return false;
+
+    handle = dlopen(fname, RTLD_LAZY | RTLD_GLOBAL);
+    if (!handle)
+	return false;
+    snprintf(fname, sizeof(fname), "gensio_init_%s", name);
+    initfunc = (int (*)(struct gensio_os_funcs *)) dlsym(handle, fname);
+    if (!initfunc) {
+	dlclose(handle);
+	return false;
+    }
+    err = initfunc(o);
+    if (err) {
+	dlclose(handle);
+	return false;
+    }
+    return true;
+}
+
+static bool
+try_loaddirs(struct gensio_os_funcs *o, const char *name, const char *paths,
+	     bool check_libtool_dir)
+{
+    char path[PATH_MAX];
+    const char *start, *end;
+    gensiods len;
+
+    start = paths;
+    while (*start) {
+	end = strchr(start, ':');
+	if (!end) {
+	    if (try_loaddir(o, name, start, check_libtool_dir))
+		return true;
+	    return false;
+	}
+
+	len = end - start;
+	if (len < sizeof(path)) {
+	    memcpy(path, start, len);
+	    path[len] = '\0';
+	    if (try_loaddir(o, name, path, check_libtool_dir))
+		return true;
+	}
+	start = end + 1;
+    }
+    return false;
+}
+
+static const char *default_libdir = PKG_LIBEXEC;
+
+bool
+gensio_os_loadlib(struct gensio_os_funcs *o, const char *name)
+{
+    const char *paths;
+
+    paths = getenv("LD_LIBRARY_PATH");
+    if (paths) {
+	if (try_loaddirs(o, name, paths, true))
+	    return true;
+    }
+    paths = getenv("GENSIO_LIBRARY_PATH");
+    if (paths) {
+	if (try_loaddirs(o, name, paths, false))
+	    return true;
+    }
+    return try_loaddirs(o, name, default_libdir, false);
+}
+
 #endif /* _WIN32 */
 
 #ifdef ENABLE_INTERNAL_TRACE
@@ -3465,3 +3741,208 @@ gensio_i_free(struct gensio_memtrack *m, void *data)
 }
 
 #endif /* ENABLE_INTERNAL_TRACE */
+
+void
+gensio_os_funcs_set_vlog(struct gensio_os_funcs *o, gensio_vlog_func func)
+{
+    o->vlog = func;
+}
+
+gensio_vlog_func *
+gensio_os_funcs_get_vlog(struct gensio_os_funcs *o)
+{
+    return o->vlog;
+}
+
+void
+gensio_os_funcs_free(struct gensio_os_funcs *o)
+{
+    o->free_funcs(o);
+}
+
+void *
+gensio_os_funcs_zalloc(struct gensio_os_funcs *o, gensiods size)
+{
+    return o->zalloc(o, size);
+}
+
+void
+gensio_os_funcs_zfree(struct gensio_os_funcs *o, void *data)
+{
+    o->free(o, data);
+}
+
+struct gensio_waiter *
+gensio_os_funcs_alloc_waiter(struct gensio_os_funcs *o)
+{
+    return o->alloc_waiter(o);
+}
+
+void
+gensio_os_funcs_free_waiter(struct gensio_os_funcs *o,
+			    struct gensio_waiter *waiter)
+{
+    o->free_waiter(waiter);
+}
+
+int
+gensio_os_funcs_wait(struct gensio_os_funcs *o,
+		     struct gensio_waiter *waiter, unsigned int count,
+		     gensio_time *timeout)
+{
+    return o->wait(waiter, count, timeout);
+}
+
+int
+gensio_os_funcs_wait_intr(struct gensio_os_funcs *o,
+			  struct gensio_waiter *waiter, unsigned int count,
+			  gensio_time *timeout)
+{
+    return o->wait_intr(waiter, count, timeout);
+}
+
+int
+gensio_os_funcs_wait_intr_sigmask(struct gensio_os_funcs *o,
+				  struct gensio_waiter *waiter,
+				  unsigned int count,
+				  gensio_time *timeout,
+				  struct gensio_os_proc_data *proc_data)
+{
+    return o->wait_intr_sigmask(waiter, count, timeout, proc_data);
+}
+
+void
+gensio_os_funcs_wake(struct gensio_os_funcs *o,
+		     struct gensio_waiter *waiter)
+{
+    o->wake(waiter);
+}
+
+struct gensio_lock *
+gensio_os_funcs_alloc_lock(struct gensio_os_funcs *o)
+{
+    return o->alloc_lock(o);
+}
+
+void
+gensio_os_funcs_free_lock(struct gensio_os_funcs *o,
+			  struct gensio_lock *lock)
+{
+    o->free_lock(lock);
+}
+
+void
+gensio_os_funcs_lock(struct gensio_os_funcs *o,
+		     struct gensio_lock *lock)
+{
+    o->lock(lock);
+}
+
+void
+gensio_os_funcs_unlock(struct gensio_os_funcs *o,
+		       struct gensio_lock *lock)
+{
+    o->unlock(lock);
+}
+
+void
+gensio_os_funcs_get_monotonic_time(struct gensio_os_funcs *o,
+				   gensio_time *time)
+{
+    o->get_monotonic_time(o, time);
+}
+
+struct gensio_timer *
+gensio_os_funcs_alloc_timer(struct gensio_os_funcs *o,
+			    void (*handler)(struct gensio_timer *t,
+					    void *cb_data),
+			    void *cb_data)
+{
+    return o->alloc_timer(o, handler, cb_data);
+}
+
+void
+gensio_os_funcs_free_timer(struct gensio_os_funcs *o,
+			   struct gensio_timer *timer)
+{
+    o->free_timer(timer);
+}
+
+int
+gensio_os_funcs_start_timer(struct gensio_os_funcs *o,
+			    struct gensio_timer *timer,
+			    gensio_time *timeout)
+{
+    return o->start_timer(timer, timeout);
+}
+
+int gensio_os_funcs_start_timer_abs(struct gensio_os_funcs *o,
+				    struct gensio_timer *timer,
+				    gensio_time *timeout)
+{
+    return o->start_timer_abs(timer, timeout);
+}
+
+int
+gensio_os_funcs_stop_timer(struct gensio_os_funcs *o,
+			   struct gensio_timer *timer)
+{
+    return o->stop_timer(timer);
+}
+
+int
+gensio_os_funcs_stop_timer_with_done(struct gensio_os_funcs *o,
+				 struct gensio_timer *timer,
+				 void (*done_handler)(struct gensio_timer *t,
+						      void *cb_data),
+				 void *cb_data)
+{
+    return o->stop_timer_with_done(timer, done_handler, cb_data);
+}
+
+struct gensio_runner *
+gensio_os_funcs_alloc_runner(struct gensio_os_funcs *o,
+			     void (*handler)(struct gensio_runner *r,
+					     void *cb_data),
+			     void *cb_data)
+{
+    return o->alloc_runner(o, handler, cb_data);
+}
+
+void
+gensio_os_funcs_free_runner(struct gensio_os_funcs *o,
+			    struct gensio_runner *runner)
+{
+    o->free_runner(runner);
+}
+
+int
+gensio_os_funcs_run(struct gensio_os_funcs *o,
+		    struct gensio_runner *runner)
+{
+    return o->run(runner);
+}
+
+int
+gensio_os_funcs_service(struct gensio_os_funcs *o, gensio_time *timeout)
+{
+    return o->service(o, timeout);
+}
+
+int
+gensio_os_funcs_handle_fork(struct gensio_os_funcs *o)
+{
+    return o->handle_fork(o);
+}
+
+void
+gensio_os_funcs_set_data(struct gensio_os_funcs *o, void *data)
+{
+    o->other_data = data;
+}
+
+void *
+gensio_os_funcs_get_data(struct gensio_os_funcs *o)
+{
+    return o->other_data;
+}
