@@ -12,11 +12,8 @@
 #endif
 
 #include "config.h"
-#include <gensio/gensio_builtins.h>
 #include <gensio/gensio_err.h>
 
-#if HAVE_PTY
-
 #include <stdio.h>
 #include <stdlib.h>
 #if HAVE_PTSNAME_R
@@ -558,6 +555,8 @@ pty_control(void *handler_data, struct gensio_iod *iod, bool get,
     case GENSIO_CONTROL_RADDR_BIN:
 	if (!get)
 	    return GE_NOTSUP;
+	if (!tdata->iod)
+	    return GE_NOTREADY;
 	if (*datalen >= sizeof(int))
 	    *((int *) data) = o->iod_get_fd(tdata->iod);
 	*datalen = sizeof(int);
@@ -578,6 +577,8 @@ pty_control(void *handler_data, struct gensio_iod *iod, bool get,
 
 	if (get)
 	    return GE_NOTSUP;
+	if (!tdata->iod)
+	    return GE_NOTREADY;
 
 	c = sscanf(data, "%d:%d:%d:%d", &ws.ws_row, &ws.ws_col,
 		   &ws.ws_xpixel, &ws.ws_ypixel);
@@ -618,12 +619,13 @@ static const struct gensio_fd_ll_ops pty_fd_ll_ops = {
     .control = pty_control
 };
 
-int
-pty_gensio_alloc(const char * const argv[], const char * const args[],
+static int
+pty_gensio_alloc(const void *gdata, const char * const args[],
 		 struct gensio_os_funcs *o,
 		 gensio_event cb, void *user_data,
 		 struct gensio **new_gensio)
 {
+    const char * const *argv = gdata;
     struct pty_data *tdata = NULL;
     struct gensio *io;
     gensiods max_read_size = GENSIO_DEFAULT_BUF_SIZE;
@@ -792,7 +794,7 @@ pty_gensio_alloc(const char * const argv[], const char * const args[],
     return err;
 }
 
-int
+static int
 str_to_pty_gensio(const char *str, const char * const args[],
 		  struct gensio_os_funcs *o,
 		  gensio_event cb, void *user_data,
@@ -810,27 +812,13 @@ str_to_pty_gensio(const char *str, const char * const args[],
     return err;
 }
 
-#else
-
-#include <gensio/gensio_class.h>
-#include <gensio/gensio_builtins.h>
-
 int
-pty_gensio_alloc(const char * const argv[], const char * const args[],
-		 struct gensio_os_funcs *o,
-		 gensio_event cb, void *user_data,
-		 struct gensio **new_gensio)
+gensio_init_pty(struct gensio_os_funcs *o)
 {
-    return GE_NOTSUP;
-}
+    int rv;
 
-int
-str_to_pty_gensio(const char *str, const char * const args[],
-		  struct gensio_os_funcs *o,
-		  gensio_event cb, void *user_data,
-		  struct gensio **new_gensio)
-{
-    return GE_NOTSUP;
+    rv = register_gensio(o, "pty", str_to_pty_gensio, pty_gensio_alloc);
+    if (rv)
+	return rv;
+    return 0;
 }
-
-#endif
