@@ -13,11 +13,10 @@
 #include <gensio/gensio_ll_gensio.h>
 #include <gensio/gensio_acc_gensio.h>
 #include <gensio/argvutils.h>
-#include <gensio/gensio_builtins.h>
 
 #include "gensio_filter_relpkt.h"
 
-int
+static int
 relpkt_gensio_alloc(struct gensio *child, const char *const args[],
 		    struct gensio_os_funcs *o,
 		    gensio_event cb, void *user_data,
@@ -55,7 +54,7 @@ relpkt_gensio_alloc(struct gensio *child, const char *const args[],
     return 0;
 }
 
-int
+static int
 str_to_relpkt_gensio(const char *str, const char * const args[],
 		     struct gensio_os_funcs *o,
 		     gensio_event cb, void *user_data,
@@ -140,7 +139,7 @@ gensio_gensio_acc_relpkt_cb(void *acc_data, int op, void *data1, void *data2,
     }
 }
 
-int
+static int
 relpkt_gensio_accepter_alloc(struct gensio_accepter *child,
 			     const char * const args[],
 			     struct gensio_os_funcs *o,
@@ -181,7 +180,7 @@ relpkt_gensio_accepter_alloc(struct gensio_accepter *child,
     return err;
 }
 
-int
+static int
 str_to_relpkt_gensio_accepter(const char *str, const char * const args[],
 			      struct gensio_os_funcs *o,
 			      gensio_accepter_event cb,
@@ -201,3 +200,20 @@ str_to_relpkt_gensio_accepter(const char *str, const char * const args[],
 
     return err;
 }
+
+int
+gensio_init_relpkt(struct gensio_os_funcs *o)
+{
+    int rv;
+
+    rv = register_filter_gensio(o, "relpkt",
+				str_to_relpkt_gensio, relpkt_gensio_alloc);
+    if (rv)
+	return rv;
+    rv = register_filter_gensio_accepter(o, "relpkt",
+					 str_to_relpkt_gensio_accepter,
+					 relpkt_gensio_accepter_alloc);
+    if (rv)
+	return rv;
+    return 0;
+}
