@@ -8,11 +8,8 @@
 /* This code handles SCTP network I/O. */
 
 #include "config.h"
-#include <gensio/gensio_builtins.h>
 #include <gensio/gensio_err.h>
 
-#if HAVE_LIBSCTP
-
 #include <stdio.h>
 #include <stdlib.h>
 #include <sys/socket.h>
@@ -384,12 +381,13 @@ static const struct gensio_fd_ll_ops sctp_fd_ll_ops = {
     .read_ready = sctp_read_ready
 };
 
-int
-sctp_gensio_alloc(const struct gensio_addr *iai, const char * const args[],
+static int
+sctp_gensio_alloc(const void *gdata, const char * const args[],
 		  struct gensio_os_funcs *o,
 		  gensio_event cb, void *user_data,
 		  struct gensio **new_gensio)
 {
+    const struct gensio_addr *iai = gdata;
     struct sctp_data *tdata = NULL;
     struct gensio *io;
     gensiods max_read_size = GENSIO_DEFAULT_BUF_SIZE;
@@ -514,7 +512,7 @@ sctp_gensio_alloc(const struct gensio_addr *iai, const char * const args[],
     return err;
 }
 
-int
+static int
 str_to_sctp_gensio(const char *str, const char * const args[],
 		  struct gensio_os_funcs *o,
 		  gensio_event cb, void *user_data,
@@ -982,13 +980,14 @@ sctpna_base_acc_op(struct gensio_accepter *acc, int op,
     }
 }
 
-int
-sctp_gensio_accepter_alloc(struct gensio_addr *iai,
+static int
+sctp_gensio_accepter_alloc(const void *gdata,
 			   const char * const args[],
 			   struct gensio_os_funcs *o,
 			   gensio_accepter_event cb, void *user_data,
 			   struct gensio_accepter **accepter)
 {
+    const struct gensio_addr *iai = gdata;
     struct sctpna_data *nadata;
     gensiods max_read_size = GENSIO_DEFAULT_BUF_SIZE;
     unsigned int instreams = 1, ostreams = 1;
@@ -1077,7 +1076,7 @@ sctp_gensio_accepter_alloc(struct gensio_addr *iai,
     return err;
 }
 
-int
+static int
 str_to_sctp_gensio_accepter(const char *str, const char * const args[],
 			    struct gensio_os_funcs *o,
 			    gensio_accepter_event cb,
@@ -1097,43 +1096,17 @@ str_to_sctp_gensio_accepter(const char *str, const char * const args[],
     return err;
 }
 
-#else
-
 int
-sctp_gensio_alloc(const struct gensio_addr *iai, const char * const args[],
-		  struct gensio_os_funcs *o,
-		  gensio_event cb, void *user_data,
-		  struct gensio **new_gensio)
+gensio_init_sctp(struct gensio_os_funcs *o)
 {
-    return GE_NOTSUP;
-}
+    int rv;
 
-int
-str_to_sctp_gensio(const char *str, const char * const args[],
-		  struct gensio_os_funcs *o,
-		  gensio_event cb, void *user_data,
-		  struct gensio **new_gensio)
-{
-    return GE_NOTSUP;
+    rv = register_gensio(o, "sctp", str_to_sctp_gensio, sctp_gensio_alloc);
+    if (rv)
+	return rv;
+    rv = register_gensio_accepter(o, "sctp", str_to_sctp_gensio_accepter,
+				  sctp_gensio_accepter_alloc);
+    if (rv)
+	return rv;
+    return 0;
 }
-
-int
-sctp_gensio_accepter_alloc(struct gensio_addr *iai,
-			   const char * const args[],
-			   struct gensio_os_funcs *o,
-			   gensio_accepter_event cb, void *user_data,
-			   struct gensio_accepter **accepter)
-{
-    return GE_NOTSUP;
-}
-
-int
-str_to_sctp_gensio_accepter(const char *str, const char * const args[],
-			    struct gensio_os_funcs *o,
-			    gensio_accepter_event cb,
-			    void *user_data,
-			    struct gensio_accepter **acc)
-{
-    return GE_NOTSUP;
-}
-#endif
