@@ -7,11 +7,8 @@
 
 #include "config.h"
 
-#include <gensio/gensio_builtins.h>
 #include <gensio/gensio_err.h>
 
-#if HAVE_OPENSSL
-
 #include <stdlib.h>
 #include <limits.h>
 #include <string.h>
@@ -29,7 +26,7 @@
 
 #include "gensio_filter_ssl.h"
 
-int
+static int
 ssl_gensio_alloc(struct gensio *child, const char *const args[],
 		 struct gensio_os_funcs *o,
 		 gensio_event cb, void *user_data,
@@ -77,7 +74,7 @@ ssl_gensio_alloc(struct gensio *child, const char *const args[],
     return 0;
 }
 
-int
+static int
 str_to_ssl_gensio(const char *str, const char * const args[],
 		  struct gensio_os_funcs *o,
 		  gensio_event cb, void *user_data,
@@ -176,7 +173,7 @@ gensio_gensio_acc_ssl_cb(void *acc_data, int op, void *data1, void *data2,
     }
 }
 
-int
+static int
 ssl_gensio_accepter_alloc(struct gensio_accepter *child,
 			  const char * const args[],
 			  struct gensio_os_funcs *o,
@@ -218,7 +215,7 @@ ssl_gensio_accepter_alloc(struct gensio_accepter *child,
     return err;
 }
 
-int
+static int
 str_to_ssl_gensio_accepter(const char *str, const char * const args[],
 			   struct gensio_os_funcs *o,
 			   gensio_accepter_event cb,
@@ -238,43 +235,19 @@ str_to_ssl_gensio_accepter(const char *str, const char * const args[],
     return err;
 }
 
-#else /* HAVE_OPENSSL */
 int
-ssl_gensio_alloc(struct gensio *child, const char * const args[],
-		 struct gensio_os_funcs *o,
-		 gensio_event cb, void *user_data,
-		 struct gensio **net)
+gensio_init_ssl(struct gensio_os_funcs *o)
 {
-    return GE_NOTSUP;
-}
+    int rv;
 
-int
-str_to_ssl_gensio(const char *str, const char * const args[],
-		  struct gensio_os_funcs *o,
-		  gensio_event cb, void *user_data,
-		  struct gensio **new_gensio)
-{
-    return GE_NOTSUP;
+    rv = register_filter_gensio(o, "ssl",
+				str_to_ssl_gensio, ssl_gensio_alloc);
+    if (rv)
+	return rv;
+    rv = register_filter_gensio_accepter(o, "ssl",
+					 str_to_ssl_gensio_accepter,
+					 ssl_gensio_accepter_alloc);
+    if (rv)
+	return rv;
+    return 0;
 }
-
-int
-ssl_gensio_accepter_alloc(struct gensio_accepter *child,
-			  const char * const args[],
-			  struct gensio_os_funcs *o,
-			  gensio_accepter_event cb, void *user_data,
-			  struct gensio_accepter **accepter)
-{
-    return GE_NOTSUP;
-}
-
-int
-str_to_ssl_gensio_accepter(const char *str, const char * const args[],
-			   struct gensio_os_funcs *o,
-			   gensio_accepter_event cb,
-			   void *user_data,
-			   struct gensio_accepter **acc)
-{
-    return GE_NOTSUP;
-}
-
-#endif /* HAVE_OPENSSL */
