@@ -17,7 +17,6 @@
 
 #include <gensio/gensio.h>
 #include <gensio/gensio_osops.h>
-#include <gensio/gensio_builtins.h>
 #include <gensio/gensio_class.h>
 
 #include "utils.h"
@@ -1169,12 +1168,13 @@ setup_self(struct stdiona_data *nadata, bool console)
     return 0;
 }
 
-int
-stdio_gensio_alloc(const char * const argv[], const char * const args[],
+static int
+stdio_gensio_alloc(const void *gdata, const char * const args[],
 		   struct gensio_os_funcs *o,
 		   gensio_event cb, void *user_data,
 		   struct gensio **new_gensio)
 {
+    const char * const *argv = gdata;
     int err;
     struct stdiona_data *nadata = NULL;
     int i;
@@ -1255,7 +1255,7 @@ stdio_gensio_alloc(const char * const argv[], const char * const args[],
     return err;
 }
 
-int
+static int
 str_to_stdio_gensio(const char *str, const char * const args[],
 		    struct gensio_os_funcs *o,
 		    gensio_event cb, void *user_data,
@@ -1494,8 +1494,9 @@ gensio_acc_stdio_func(struct gensio_accepter *acc, int func, int val,
     }
 }
 
-int
-stdio_gensio_accepter_alloc(const char * const args[],
+static int
+stdio_gensio_accepter_alloc(const void *gdata,
+			    const char * const args[],
 			    struct gensio_os_funcs *o,
 			    gensio_accepter_event cb, void *user_data,
 			    struct gensio_accepter **accepter)
@@ -1562,12 +1563,27 @@ stdio_gensio_accepter_alloc(const char * const args[],
     return 0;
 }
 
-int
+static int
 str_to_stdio_gensio_accepter(const char *str, const char * const args[],
 			     struct gensio_os_funcs *o,
 			     gensio_accepter_event cb,
 			     void *user_data,
 			     struct gensio_accepter **acc)
 {
-    return stdio_gensio_accepter_alloc(args, o, cb, user_data, acc);
+    return stdio_gensio_accepter_alloc(NULL, args, o, cb, user_data, acc);
+}
+
+int
+gensio_init_stdio(struct gensio_os_funcs *o)
+{
+    int rv;
+
+    rv = register_gensio(o, "stdio", str_to_stdio_gensio, stdio_gensio_alloc);
+    if (rv)
+	return rv;
+    rv = register_gensio_accepter(o, "stdio", str_to_stdio_gensio_accepter,
+				  stdio_gensio_accepter_alloc);
+    if (rv)
+	return rv;
+    return 0;
 }
