@@ -18,7 +18,6 @@
 #include <gensio/gensio_class.h>
 #include <gensio/argvutils.h>
 #include <gensio/gensio_osops.h>
-#include <gensio/gensio_builtins.h>
 #include <gensio/gensio_list.h>
 
 #ifdef ENABLE_INTERNAL_TRACE
@@ -1640,7 +1639,8 @@ gensio_acc_udp_func(struct gensio_accepter *acc, int func, int val,
 }
 
 static int
-i_udp_gensio_accepter_alloc(struct gensio_addr *iai, gensiods max_read_size,
+i_udp_gensio_accepter_alloc(const struct gensio_addr *iai,
+			    gensiods max_read_size,
 			    bool reuseaddr, struct gensio_os_funcs *o,
 			    gensio_accepter_event cb, void *user_data,
 			    struct gensio_accepter **accepter)
@@ -1700,13 +1700,14 @@ i_udp_gensio_accepter_alloc(struct gensio_addr *iai, gensiods max_read_size,
     return GE_NOMEM;
 }
 
-int
-udp_gensio_accepter_alloc(struct gensio_addr *iai,
+static int
+udp_gensio_accepter_alloc(const void *gdata,
 			  const char * const args[],
 			  struct gensio_os_funcs *o,
 			  gensio_accepter_event cb, void *user_data,
 			  struct gensio_accepter **accepter)
 {
+    const struct gensio_addr *iai = gdata;
     gensiods max_read_size = GENSIO_DEFAULT_UDP_BUF_SIZE;
     unsigned int i;
     bool reuseaddr = false;
@@ -1727,7 +1728,7 @@ udp_gensio_accepter_alloc(struct gensio_addr *iai,
 				       o, cb, user_data, accepter);
 }
 
-int
+static int
 str_to_udp_gensio_accepter(const char *str, const char * const args[],
 			   struct gensio_os_funcs *o,
 			   gensio_accepter_event cb,
@@ -1747,12 +1748,13 @@ str_to_udp_gensio_accepter(const char *str, const char * const args[],
     return err;
 }
 
-int
-udp_gensio_alloc(const struct gensio_addr *addr, const char * const args[],
+static int
+udp_gensio_alloc(const void *gdata, const char * const args[],
 		 struct gensio_os_funcs *o,
 		 gensio_event cb, void *user_data,
 		 struct gensio **new_gensio)
 {
+    const struct gensio_addr *addr = gdata;
     struct udpn_data *ndata = NULL;
     struct gensio_accepter *accepter;
     struct udpna_data *nadata = NULL;
@@ -1940,7 +1942,7 @@ udp_gensio_alloc(const struct gensio_addr *addr, const char * const args[],
     return err;
 }
 
-int
+static int
 str_to_udp_gensio(const char *str, const char * const args[],
 		  struct gensio_os_funcs *o,
 		  gensio_event cb, void *user_data,
@@ -1957,3 +1959,18 @@ str_to_udp_gensio(const char *str, const char * const args[],
     gensio_addr_free(addr);
     return err;
 }
+
+int
+gensio_init_udp(struct gensio_os_funcs *o)
+{
+    int rv;
+
+    rv = register_gensio(o, "udp", str_to_udp_gensio, udp_gensio_alloc);
+    if (rv)
+	return rv;
+    rv = register_gensio_accepter(o, "udp", str_to_udp_gensio_accepter,
+				  udp_gensio_accepter_alloc);
+    if (rv)
+	return rv;
+    return 0;
+}
