@@ -13,6 +13,7 @@
 
 #include <gensio/gensio_unix.h>
 #include <gensio/selector.h>
+#include <gensio/gensio_selector.h>
 #include <gensio/gensio.h>
 #include <gensio/gensio_osops_addrinfo.h>
 #include <gensio/gensio_osops_stdsock.h>
@@ -1667,8 +1668,9 @@ gensio_unix_alloc_sel(struct selector_s *sel, int wake_sig)
 struct gensio_os_proc_data {
     struct gensio_os_funcs *o;
     int wake_sig;
-    sigset_t old_sigs;
-    sigset_t wait_sigs;
+    sigset_t old_sigs; /* Original signal mask. */
+    sigset_t wait_sigs; /* Signal mask to use when waiting. */
+    sigset_t check_sigs; /* Signals we are checking for. */
 
     struct sigaction old_wakesig;
 
@@ -1701,6 +1703,8 @@ struct gensio_os_proc_data {
     void *winch_handler_data;
     int winch_fd;
 #endif
+
+    struct gensio_os_cleanup_handler *cleanup_handlers;
 };
 
 /* We only have one of these per process, so it's global. */
@@ -1752,6 +1756,7 @@ gensio_os_proc_setup(struct gensio_os_funcs *o,
 	data->wake_sig = o->get_wake_sig(o);
 
     sigemptyset(&sigs);
+    sigemptyset(&data->check_sigs);
     if (data->wake_sig)
 	sigaddset(&sigs, data->wake_sig);
     sigaddset(&sigs, SIGCHLD); /* Ignore SIGCHLD in normal operation. */
@@ -1765,6 +1770,7 @@ gensio_os_proc_setup(struct gensio_os_funcs *o,
     if (data->wake_sig)
 	sigdelset(&data->wait_sigs, data->wake_sig);
     sigdelset(&data->wait_sigs, SIGCHLD); /* Allow SIGCHLD while waiting. */
+    sigaddset(&data->check_sigs, SIGCHLD);
     sigaddset(&data->wait_sigs, SIGPIPE); /* No SIGPIPE ever. */
 
     memset(&sigdo, 0, sizeof(sigdo));
@@ -1804,9 +1810,63 @@ gensio_os_proc_setup(struct gensio_os_funcs *o,
     return 0;
 }
 
+void
+gensio_register_os_cleanup_handler(struct gensio_os_funcs *o,
+				   struct gensio_os_cleanup_handler *h)
+{
+    struct gensio_data *d = o->user_data;
+    struct gensio_os_proc_data *data = d->pdata;
+
+    LOCK(&data->handler_lock);
+    h->next = data->cleanup_handlers;
+    data->cleanup_handlers = h;
+    UNLOCK(&data->handler_lock);
+}
+
+static int
+check_for_sigpending(sigset_t *check_for)
+{
+#ifdef HAVE_SIGTIMEDWAIT
+    static struct timespec zerotime = { 0, 0 };
+    return sigtimedwait(check_for, NULL, &zerotime);
+#else
+#ifdef NSIG
+    /* Nothing to do */
+#elif defined(_NSIG)
+#define NSIG _NSIG
+#elif defined(SIGRTMAX)
+#define NSIG SIGRTMAX
+#else
+#define NSIG 64
+#endif
+    int rsig = 0, sig;
+    sigset_t pending;
+
+    sigpending(&pending);
+    for (sig = 0; sig < NSIG; sig++) {
+	if (sigismember(&pending, sig) && sigismember(check_for, sig)) {
+	    rsig = sig;
+	    sigwait(&check_for, &sig);
+	    break;
+	}
+    }
+    return rsig;
+#endif
+}
+
 void
 gensio_os_proc_cleanup(struct gensio_os_proc_data *data)
 {
+    int sig;
+
+    /* We should be single-threaded here. */
+    while (data->cleanup_handlers) {
+	struct gensio_os_cleanup_handler *h = data->cleanup_handlers;
+
+	data->cleanup_handlers = h->next;
+	h->cleanup(h);
+    }
+
     LOCK_DESTROY(&data->handler_lock);
 
     if (data->wake_sig)
@@ -1828,13 +1888,48 @@ gensio_os_proc_cleanup(struct gensio_os_proc_data *data)
     }
 #endif
     sigaction(SIGCHLD, &data->old_sigchld, NULL);
+
+    /* Clear out any pending signals before we restore the mask. */
+    while ((sig = check_for_sigpending(&data->check_sigs)) > 0)
+	;
+
     sigprocmask(SIG_SETMASK, &data->old_sigs, NULL);
 }
 
 void
 gensio_os_proc_check_handlers(struct gensio_os_proc_data *data)
 {
+    int sig;
+
     LOCK(&data->handler_lock);
+    /*
+     * Poll implementations (at least epoll) will not necessarily
+     * check for signals if they return immediately.  So to avoid
+     * missing a signal on a really busy system, check for signals
+     * here.
+     */
+    while ((sig = check_for_sigpending(&data->check_sigs)) > 0) {
+	switch(sig) {
+	case SIGCHLD:
+	    data->got_sigchld = true;
+	    break;
+	case SIGQUIT:
+	case SIGTERM:
+	case SIGINT:
+	    data->got_term_sig = true;
+	    break;
+	case SIGHUP:
+	    data->got_reload_sig = true;
+	    break;
+#if HAVE_DECL_SIGWINCH
+	case SIGWINCH:
+	    data->got_winch_sig = true;
+	    break;
+#endif
+	default:
+	    assert(0);
+	}
+    }
     if (data->got_term_sig) {
 	data->got_term_sig = false;
 	data->term_handler(data->term_handler_data);
@@ -1913,6 +2008,9 @@ gensio_os_proc_register_term_handler(struct gensio_os_proc_data *data,
     sigdelset(&data->wait_sigs, SIGINT);
     sigdelset(&data->wait_sigs, SIGQUIT);
     sigdelset(&data->wait_sigs, SIGTERM);
+    sigaddset(&data->check_sigs, SIGINT);
+    sigaddset(&data->check_sigs, SIGQUIT);
+    sigaddset(&data->check_sigs, SIGTERM);
     data->term_sig_set = true;
     return 0;
 
@@ -1953,6 +2051,7 @@ int gensio_os_proc_register_reload_handler(struct gensio_os_proc_data *data,
 	goto out_err;
     }
     sigdelset(&data->wait_sigs, SIGHUP);
+    sigaddset(&data->check_sigs, SIGHUP);
     data->reload_sig_set = true;
     return 0;
 
