@@ -8,7 +8,6 @@
 #include "config.h"
 
 #include <gensio/sergensio_class.h>
-#include <gensio/gensio_builtins.h>
 
 #include "utils.h"
 #include "gensio_ll_ipmisol.h"
@@ -53,12 +52,13 @@ sergensio_iterm_func(struct sergensio *sio, int op, int val, char *buf,
     return idata->ops(idata->ll, op, val, buf, done, cb_data);
 }
 
-int
-ipmisol_gensio_alloc(const char *devname, const char * const args[],
+static int
+ipmisol_gensio_alloc(const void *gdata, const char * const args[],
 		     struct gensio_os_funcs *o,
 		     gensio_event cb, void *user_data,
 		     struct gensio **rio)
 {
+    const char *devname = gdata;
     struct iterm_data *idata;
     int err;
     gensiods max_read_size = GENSIO_DEFAULT_BUF_SIZE;
@@ -109,7 +109,7 @@ ipmisol_gensio_alloc(const char *devname, const char * const args[],
     return err;
 }
 
-int
+static int
 str_to_ipmisol_gensio(const char *str, const char * const args[],
 		      struct gensio_os_funcs *o,
 		      gensio_event cb, void *user_data,
@@ -117,3 +117,15 @@ str_to_ipmisol_gensio(const char *str, const char * const args[],
 {
     return ipmisol_gensio_alloc(str, args, o, cb, user_data, new_gensio);
 }
+
+int
+gensio_init_ipmisol(struct gensio_os_funcs *o)
+{
+    int rv;
+
+    rv = register_gensio(o, "ipmisol", str_to_ipmisol_gensio,
+			 ipmisol_gensio_alloc);
+    if (rv)
+	return rv;
+    return 0;
+}
