@@ -16,7 +16,6 @@
 
 #include <gensio/sergensio_class.h>
 #include <gensio/gensio_ll_fd.h>
-#include <gensio/gensio_builtins.h>
 #include <gensio/gensio_osops.h>
 
 #include "uucplock.h"
@@ -577,7 +576,7 @@ serialdev_timeout(struct gensio_timer *t, void *cb_data)
     bool force_send;
 
     sterm_lock(sdata);
-    if (sdata->handling_modemstate) {
+    if (sdata->handling_modemstate || !sdata->open) {
 	sterm_unlock(sdata);
 	return;
     }
@@ -772,6 +771,9 @@ sterm_check_close_drain(void *handler_data, struct gensio_iod *iod,
     if (!sdata->timer_stopped)
 	goto out_einprogress;
 
+    if (sdata->handling_modemstate)
+	goto out_einprogress;
+
     rv = o->bufcount(sdata->iod, GENSIO_OUT_BUF, &count);
     if (rv || count <= 0)
 	goto out_rm_uucp;
@@ -808,6 +810,9 @@ sterm_check_close_drain(void *handler_data, struct gensio_iod *iod,
 	    uucp_rm_lock(sdata->devname);
 	gensio_fd_ll_close_now(sdata->ll);
     }
+    if (err != GE_INPROGRESS)
+	/* We are really closing, the iod will be destroyed now. */
+	sdata->iod = NULL;
  out_unlock:
     sterm_unlock(sdata);
     return err;
@@ -1375,12 +1380,13 @@ sergensio_setup_defaults(struct sterm_data *sdata)
     return 0;
 }
 
-int
-serialdev_gensio_alloc(const char *devname, const char * const args[],
+static int
+serialdev_gensio_alloc(const void *gdata, const char * const args[],
 		       struct gensio_os_funcs *o,
 		       gensio_event cb, void *user_data,
 		       struct gensio **rio)
 {
+    const char *devname = gdata;
     struct sterm_data *sdata = o->zalloc(o, sizeof(*sdata));
     struct gensio *io;
     int err;
@@ -1528,7 +1534,7 @@ serialdev_gensio_alloc(const char *devname, const char * const args[],
     return err;
 }
 
-int
+static int
 str_to_serialdev_gensio(const char *str, const char * const args[],
 		      struct gensio_os_funcs *o,
 		      gensio_event cb, void *user_data,
@@ -1536,3 +1542,15 @@ str_to_serialdev_gensio(const char *str, const char * const args[],
 {
     return serialdev_gensio_alloc(str, args, o, cb, user_data, new_gensio);
 }
+
+int
+gensio_init_serialdev(struct gensio_os_funcs *o)
+{
+    int rv;
+
+    rv = register_gensio(o, "serialdev", str_to_serialdev_gensio,
+			 serialdev_gensio_alloc);
+    if (rv)
+	return rv;
+    return 0;
+}
