@@ -14,7 +14,6 @@
 #include <gensio/sergensio_class.h>
 #include <gensio/gensio_ll_gensio.h>
 #include <gensio/gensio_acc_gensio.h>
-#include <gensio/gensio_builtins.h>
 
 #include "utils.h"
 #include "telnet.h"
@@ -968,7 +967,7 @@ stel_setup(const char * const args[], bool default_is_client,
     return err;
 }
 
-int
+static int
 telnet_gensio_alloc(struct gensio *child, const char * const args[],
 		    struct gensio_os_funcs *o,
 		    gensio_event cb, void *user_data,
@@ -1024,7 +1023,7 @@ telnet_gensio_alloc(struct gensio *child, const char * const args[],
     return err;
 }
 
-int
+static int
 str_to_telnet_gensio(const char *str, const char * const args[],
 		     struct gensio_os_funcs *o,
 		     gensio_event cb, void *user_data,
@@ -1193,7 +1192,7 @@ sergensio_stela_func(struct sergensio_accepter *sacc,
     return GE_NOTSUP;
 }
 
-int
+static int
 telnet_gensio_accepter_alloc(struct gensio_accepter *child,
 			     const char * const args[],
 			     struct gensio_os_funcs *o,
@@ -1266,7 +1265,7 @@ telnet_gensio_accepter_alloc(struct gensio_accepter *child,
     return err;
 }
 
-int
+static int
 str_to_telnet_gensio_accepter(const char *str, const char * const args[],
 			      struct gensio_os_funcs *o,
 			      gensio_accepter_event cb,
@@ -1285,3 +1284,20 @@ str_to_telnet_gensio_accepter(const char *str, const char * const args[],
 
     return err;
 }
+
+int
+gensio_init_telnet(struct gensio_os_funcs *o)
+{
+    int rv;
+
+    rv = register_filter_gensio(o, "telnet",
+				str_to_telnet_gensio, telnet_gensio_alloc);
+    if (rv)
+	return rv;
+    rv = register_filter_gensio_accepter(o, "telnet",
+					 str_to_telnet_gensio_accepter,
+					 telnet_gensio_accepter_alloc);
+    if (rv)
+	return rv;
+    return 0;
+}
