@@ -72,14 +72,14 @@ class EventHandler:
         """
         return
 
-    def new_channel(self, old_io, new_io):
+    def new_channel(self, old_io, new_io, auxdata):
         """A new channel has been requested from the other end.
 
         old_io - an existing channel on a mux gensio
         new_io - a new channel for the mux gensio
 
         Return an integer. If you return 0, the new channel is
-        accepted.  If you return non-zero, it is considered a gensio
+        accepted.  If you return a GE_xxx erro, it is considered a gensio
         error and that error is reported as an open channel failure on
         the remote end.  If you return non-zero, the new_io is freed,
         you should not use it.
@@ -209,8 +209,8 @@ class EventHandler:
         requested.
 
         sio -- The sergensio object requesting the flush.
-        val -- The flush type, one of SERGIO_FLUSH_RCV_BUFFER,
-            SERGIO_FLUSH_XMIT_BUFFER, or SERGIO_FLUSH_RCV_XMIT_BUFFERS.
+        val -- The flush type, one of SERGENSIO_FLUSH_RCV_BUFFER,
+            SERGENSIO_FLUSH_XMIT_BUFFER, or SERGENSIO_FLUSH_RCV_XMIT_BUFFERS.
         """
         return
 
@@ -460,7 +460,7 @@ class gensio:
         """
         return
 
-    def open_channel(self, args, handler, done):
+    def alloc_channel(self, args, handler):
         """Create a new channel on the gensio.  This is gensio-specific, most
         gensios don't support channels.
 
@@ -471,13 +471,6 @@ class gensio:
         """
         return
 
-    def open_channel_s(self, args, handler):
-        """Like open_channel(), but waits for the open to complete and raises
-        an exception on an error.  The wait operation is a normal
-        waiter, so gensio is fully operational during the wait.
-        """
-        return
-
     def get_type(self):
         """Return the type string for the gensio."""
         return
@@ -646,8 +639,18 @@ class gensio:
         """
         return None
 
+    def same_as(self, other):
+        """Compare two gensios to see if they are the same.
+
+        Due to the way python works, you can hold two different
+        pointers to the same gensio.  This compares the base data and
+        returns True if two gensio object are the same, False if not.
+        """
+        return False
+
 class SergensioDone:
-    """These are methods called when a sergensio request from a client
+
+        """These are methods called when a sergensio request from a client
     completes.  Note that the base code may not honor the request or
     may choose a different value than requested.  Check the returned
     value to be sure.  Also, an error may occur, if the err value is
@@ -812,8 +815,8 @@ class sergensio:
     def sg_flush(self, val):
         """Request that the remote end do a data flush.
 
-        val -- The flush type, one of SERGIO_FLUSH_RCV_BUFFER,
-            SERGIO_FLUSH_XMIT_BUFFER, or SERGIO_FLUSH_RCV_XMIT_BUFFERS.
+        val -- The flush type, one of SERGENSIO_FLUSH_RCV_BUFFER,
+            SERGENSIO_FLUSH_XMIT_BUFFER, or SERGENSIO_FLUSH_RCV_XMIT_BUFFERS.
         """
         return
 
@@ -1288,3 +1291,84 @@ def gensio_get_log_mask():
     """Return the current log mask.  See gensio_set_log_mask() above
     for details."""
     return 0
+
+class ifinfo:
+    def __init__(o):
+        """Get information about all the network interafces on the box and
+        return them in an object.
+
+        o -- The gensio_os_funcs to use for this.
+        """
+        return
+
+    def get_num_ifs():
+        """Return the number of interfaces found, used for indexing"""
+        return
+
+    def get_name(idx):
+        """Return the name of interface as a string.
+
+        idx -- The index of the interface in the list (NOT the ifindex).
+        """
+        return
+
+    def is_up(idx):
+        """Is the given index up?  Returns a bool.
+
+        idx -- The index of the interface in the list (NOT the ifindex).
+        """
+        return
+
+    def is_loopback(idx):
+        """Is the given index a loopback?  Returns a bool.
+
+        idx -- The index of the interface in the list (NOT the ifindex).
+        """
+        return
+
+    def is_multicast(idx):
+        """Is the given index multicast capable?  Returns a bool.
+
+        idx -- The index of the interface in the list (NOT the ifindex).
+        """
+        return
+
+    def get_ifindex(idx):
+        """Return the system interface index of the interface, use for other
+        system operations.  This is not the same as "idx".  Returns an
+        integer.
+
+        idx -- The index of the interface in the list (NOT the ifindex).
+
+        """
+        return
+
+    def get_num_addrs(idx):
+        """Return the number of IP addresses on this interface.  Returns an
+        integer.
+
+        idx -- The index of the interface in the list (NOT the ifindex).
+
+        """
+        return
+
+    def get_addr_netbits(idx, addridx):
+        """Return the number of of network bits the given address.  This is
+        the same as the number after a slash in an address, like
+        127.0.0.1/8.  Returns an integer.
+
+        idx -- The index of the interface in the list (NOT the ifindex).
+        addridx -- The index of the address in the interface.
+
+        """
+        return
+
+    def get_addr(idx, addridx):
+        """Return the number of of network bits the given address.  Returns a
+        string in the form: "ipv4:n.n.n.n" or "ipv6:::1".
+
+        idx -- The index of the interface in the list (NOT the ifindex).
+        addridx -- The index of the address in the interface.
+
+        """
+        return
