@@ -40,6 +40,7 @@
 #include <gensio/gensio_osops.h>
 #include <gensio/gensio_osops_addrinfo.h>
 #include <gensio/gensio_osops_stdsock.h>
+#include <gensio/argvutils.h>
 
 #define TCL_THREADS 1
 #include <tcl.h>
@@ -53,6 +54,7 @@
 #include <sys/wait.h>
 #include <sys/uio.h>
 #include <string.h>
+#include <sys/ioctl.h>
 
 struct gensio_data
 {
@@ -124,6 +126,7 @@ struct gensio_iod_tcl {
     bool in_clear;
     enum { CL_NOT_CALLED, CL_CALLED, CL_DONE } close_state;
 
+    int orig_fd;
     int fd;
     enum gensio_iod_type type;
     void *sockinfo;
@@ -144,6 +147,12 @@ struct gensio_iod_tcl {
     bool read_enabled;
     bool write_enabled;
     bool in_handler;
+
+    /* For GENSIO_IOD_PTY */
+    const char **argv;
+    const char **env;
+    char *start_dir;
+    int pid;
 };
 
 #define i_to_tcl(i) gensio_container_of(i, struct gensio_iod_tcl, r);
@@ -871,11 +880,11 @@ gensio_tcl_wake(struct gensio_waiter *w)
 
 static int
 gensio_tcl_add_iod(struct gensio_os_funcs *o, enum gensio_iod_type type,
-		    intptr_t fd, struct gensio_iod **riod)
+		   intptr_t ofd, struct gensio_iod **riod)
 {
     struct gensio_iod_tcl *iod = NULL;
     bool closefd = false;
-    int err = GE_NOMEM;
+    int err = GE_NOMEM, fd = ofd;
 
     if (type == GENSIO_IOD_CONSOLE) {
        if (fd == 0)
@@ -887,6 +896,11 @@ gensio_tcl_add_iod(struct gensio_os_funcs *o, enum gensio_iod_type type,
        if (fd == -1)
            return gensio_os_err_to_err(o, errno);
        closefd = true;
+    } else if (type == GENSIO_IOD_PTY) {
+	err = gensio_unix_pty_alloc(o, &fd);
+	if (err)
+	    return err;
+	closefd = true;
     }
 
     iod = o->zalloc(o, sizeof(*iod));
@@ -897,6 +911,7 @@ gensio_tcl_add_iod(struct gensio_os_funcs *o, enum gensio_iod_type type,
 
     iod->r.f = o;
     iod->fd = fd;
+    iod->orig_fd = ofd;
     if (type == GENSIO_IOD_STDIO) {
 	struct stat statb;
 
@@ -916,6 +931,8 @@ gensio_tcl_add_iod(struct gensio_os_funcs *o, enum gensio_iod_type type,
 	    err = GE_INVAL;
 	    goto out_err;
 	}
+    } else if (type == GENSIO_IOD_PTY) {
+	iod->pid = -1;
     }
     iod->type = type;
 
@@ -939,13 +956,20 @@ gensio_tcl_add_iod(struct gensio_os_funcs *o, enum gensio_iod_type type,
 static void
 gensio_tcl_release_iod(struct gensio_iod *iiod)
 {
+    struct gensio_os_funcs *o = iiod->f;
     struct gensio_iod_tcl *iod = i_to_tcl(iiod);
 
     assert(!iod->handlers_set);
     Tcl_MutexFinalize(&iod->lock);
     if (iod->type == GENSIO_IOD_FILE)
-	iod->r.f->free_runner(iod->runner);
-    iiod->f->free(iiod->f, iod);
+	o->free_runner(iod->runner);
+    if (iod->type == GENSIO_IOD_PTY) {
+	if (iod->argv)
+	    gensio_argv_free(o, iod->argv);
+	if (iod->env)
+	    gensio_argv_free(o, iod->env);
+    }
+    o->free(o, iod);
 }
 
 static int
@@ -964,6 +988,87 @@ gensio_tcl_iod_get_fd(struct gensio_iod *iiod)
     return iod->fd;
 }
 
+static int
+gensio_tcl_pty_control(struct gensio_iod_tcl *iod, int op, bool get,
+		       intptr_t val)
+{
+    struct gensio_os_funcs *o = iod->r.f;
+    int err = 0;
+    const char **nargv;
+
+    if (get) {
+	if (op == GENSIO_IOD_CONTROL_PID) {
+	    if (iod->pid == -1)
+		return GE_NOTREADY;
+	    *((intptr_t *) val) = iod->pid;
+	    return 0;
+	}
+	return GE_NOTSUP;
+    }
+
+    switch (op) {
+    case GENSIO_IOD_CONTROL_ARGV:
+	err = gensio_argv_copy(o, (const char **) val, NULL, &nargv);
+	if (err)
+	    return err;
+	if (iod->argv)
+	    gensio_argv_free(o, iod->argv);
+	iod->argv = nargv;
+	return 0;
+
+    case GENSIO_IOD_CONTROL_ENV:
+	err = gensio_argv_copy(o, (const char **) val, NULL, &nargv);
+	if (err)
+	    return err;
+	if (iod->env)
+	    gensio_argv_free(o, iod->env);
+	iod->env = nargv;
+	return 0;
+
+    case GENSIO_IOD_CONTROL_START:
+	return gensio_unix_pty_start(o, iod->fd, iod->argv,
+				     iod->env, iod->start_dir, &iod->pid);
+
+    case GENSIO_IOD_CONTROL_STOP:
+	if (iod->fd != -1) {
+	    close(iod->fd);
+	    iod->fd = -1;
+	}
+	return 0;
+
+    case GENSIO_IOD_CONTROL_WIN_SIZE: {
+	struct winsize win;
+	struct gensio_winsize *gwin = (struct gensio_winsize *) val;
+
+	win.ws_row = gwin->ws_row;
+	win.ws_col = gwin->ws_col;
+	win.ws_xpixel = gwin->ws_xpixel;
+	win.ws_ypixel = gwin->ws_ypixel;
+	if (ioctl(iod->fd, TIOCSWINSZ, &win) == -1)
+	    err = gensio_os_err_to_err(o, errno);
+	return err;
+    }
+
+    case GENSIO_IOD_CONTROL_START_DIR: {
+	char *dir = (char *) val;
+
+	if (dir) {
+	    dir = gensio_strdup(o, dir);
+	    if (!dir)
+		return GE_NOMEM;
+	}
+
+	if (iod->start_dir)
+	    o->free(o, iod->start_dir);
+	iod->start_dir = dir;
+	return 0;
+    }
+
+    default:
+	return GE_NOTSUP;
+    }
+}
+
 static int
 gensio_tcl_iod_control(struct gensio_iod *iiod, int op, bool get, intptr_t val)
 {
@@ -981,6 +1086,9 @@ gensio_tcl_iod_control(struct gensio_iod *iiod, int op, bool get, intptr_t val)
 	return 0;
     }
 
+    if (iod->type == GENSIO_IOD_PTY)
+	return gensio_tcl_pty_control(iod, op, get, val);
+
     if (iod->type != GENSIO_IOD_DEV)
 	return GE_NOTSUP;
 
@@ -1029,13 +1137,15 @@ gensio_tcl_close(struct gensio_iod **iodp)
 		iod->close_state = CL_DONE;
 	}
     } else if (!iod->is_stdio) {
-	err = close(iod->fd);
-	if (err == -1)
-	    err = gensio_os_err_to_err(o, errno);
+	if (iod->fd != -1) {
+	    err = close(iod->fd);
+	    if (err == -1)
+		err = gensio_os_err_to_err(o, errno);
 #ifdef ENABLE_INTERNAL_TRACE
 	/* Close should never fail, but don't crash in production builds. */
-	assert(err == 0);
+	    assert(err == 0);
 #endif
+	}
     }
     o->release_iod(iiod);
     *iodp = NULL;
@@ -1135,7 +1245,7 @@ gensio_tcl_makeraw(struct gensio_iod *iiod)
 {
     struct gensio_iod_tcl *iod = i_to_tcl(iiod);
 
-    if (iod->fd == 1 || iod->fd == 2 || iod->type == GENSIO_IOD_FILE)
+    if (iod->orig_fd == 1 || iod->orig_fd == 2 || iod->type == GENSIO_IOD_FILE)
 	/* Only set this for stdin or other files. */
 	return 0;
 
@@ -1174,7 +1284,8 @@ generic_close(intptr_t fd)
 static int
 gensio_tcl_exec_subprog(struct gensio_os_funcs *o,
 			const char *argv[], const char **env,
-			bool stderr_to_stdout,
+			const char *start_dir,
+			unsigned int flags,
 			intptr_t *rpid,
 			struct gensio_iod **rstdin,
 			struct gensio_iod **rstdout,
@@ -1188,7 +1299,7 @@ gensio_tcl_exec_subprog(struct gensio_os_funcs *o,
     int uinfd = -1, uoutfd = -1, uerrfd = -1;
     int upid = -1;
 
-    err = gensio_unix_do_exec(o, argv, env, stderr_to_stdout, &upid, &uinfd,
+    err = gensio_unix_do_exec(o, argv, env, start_dir, flags, &upid, &uinfd,
 			      &uoutfd, rstderr ? &uerrfd : NULL);
     if (err)
 	return err;
