@@ -9,8 +9,9 @@ from utils import *
 import gensio
 
 print("Test accept certauth-ssl-tcp")
-ta = TestAccept(o, "certauth(cert=%s/clientcert.pem,key=%s/clientkey.pem,username=testuser,service=myservice),ssl(CA=%s/CA.pem),tcp,localhost," % (keydir, keydir, keydir), "certauth(CA=%s/clientcert.pem),ssl(key=%s/key.pem,cert=%s/cert.pem),tcp,0" % (keydir, keydir, keydir), do_test, do_close = False)
-cn = ta.io2.control(0, True, gensio.GENSIO_CONTROL_GET_PEER_CERT_NAME,
+ta = TestAccept(o, "certauth(cert=%s/clientcert.pem,key=%s/clientkey.pem,username=testuser,service=myservice),ssl(CA=%s/CA.pem),tcp,localhost," % (keydir, keydir, keydir), "certauth(CA=%s/clientcert.pem),ssl(key=%s/key.pem,cert=%s/cert.pem),tcp,ipv4,0" % (keydir, keydir, keydir), do_test, do_close = False)
+cn = ta.io2.control(0, gensio.GENSIO_CONTROL_GET,
+                    gensio.GENSIO_CONTROL_GET_PEER_CERT_NAME,
                     "-1,CN");
 i = cn.index(',')
 cn2 = cn[i+1:]
@@ -22,12 +23,17 @@ if cn2[i+1:] != "gensio.org":
     raise Exception(
         "Invalid common name in certificate, expected %s, got %s" %
         ("gensio.org", cn2[i+1:]))
-username = ta.io2.control(0, True, gensio.GENSIO_CONTROL_USERNAME, None)
+username = ta.io2.control(0, gensio.GENSIO_CONTROL_GET,
+                          gensio.GENSIO_CONTROL_USERNAME, None)
 if username != "testuser":
     raise Exception(
         "Invalid username, expected %s, got %s" % ("testuser", username))
-service = ta.io2.control(0, True, gensio.GENSIO_CONTROL_SERVICE, None)
+service = ta.io2.control(0, gensio.GENSIO_CONTROL_GET,
+                         gensio.GENSIO_CONTROL_SERVICE, None)
 if service != "myservice":
     raise Exception(
         "Invalid service, expected %s, got %s" % ("myservice", service))
 ta.close()
+del ta
+del o
+test_shutdown()
