@@ -13,7 +13,7 @@ check_echo_dev()
 io = alloc_io(o, "serialdev," + ttyecho + ",38400")
 check_raddr(io, "echo device", ttyecho + ",38400N81 RTSHI DTRHI")
 test_dataxfer(io, io, "This is a test string!")
-io_close(io)
+io_close([io])
 del io
 del o
 test_shutdown()
