@@ -19,6 +19,8 @@ def do_stream_test(io1, io2):
     print("  Success!")
 
 print("Test sctp streams")
-ta = TestAccept(o, "sctp(instreams=2,ostreams=3),localhost,",
-                "sctp(instreams=3,ostreams=2),0", do_stream_test,
-                chunksize = 64)
+TestAccept(o, "sctp(instreams=2,ostreams=3),localhost,",
+           "sctp(instreams=3,ostreams=2),0", do_stream_test,
+           chunksize = 64)
+del o
+test_shutdown()
