@@ -104,7 +104,8 @@ class HandleData:
         self.debug = 0
         return
 
-    def set_compare(self, to_compare, start_reader = True, stream = None):
+    def set_compare(self, to_compare, start_reader = True, stream = None,
+                    auxdata = None):
         """Set some data to compare
 
         If start_reader is true (default), it enable the read callback.
@@ -112,12 +113,14 @@ class HandleData:
         """
         self.compared = 0
         self.stream = stream
+        self.compare_auxdata = auxdata
         self.to_compare = conv_to_bytes(to_compare);
         if (start_reader):
             self.io.read_cb_enable(True)
         return
 
-    def set_compare_oob(self, to_compare, start_reader = True, stream = None):
+    def set_compare_oob(self, to_compare, start_reader = True, stream = None,
+                        auxdata = None):
         """Set some oob data to compare
 
         If start_reader is true (default), it enable the read callback.
@@ -125,6 +128,7 @@ class HandleData:
         """
         self.compared_oob = 0
         self.stream = stream
+        self.compare_auxdata = auxdata
         self.to_compare_oob = conv_to_bytes(to_compare)
         if (start_reader):
             self.io.read_cb_enable(True)
@@ -215,6 +219,21 @@ class HandleData:
         oob = False;
         stream = 0
 
+        if (self.compare_auxdata is not None):
+            if not auxdata:
+                raise HandlerException("%s: no auxdata, expected %s" %
+                                       (self.name, str(self.compare_auxdata)))
+            if len(auxdata) != len(self.compare_auxdata):
+                raise HandlerException(
+                    "%s: length mismatch in auxdata, expected %s, got %s" %
+                    (self.name, str(self.compare_auxdata), str(auxdata)))
+
+            for i in range(0, len(auxdata)):
+                if auxdata[i] != self.compare_auxdata[i]:
+                    raise HandlerException(
+                        "%s: auxdata item %d wrong, expected %s, got %s" %
+                        (self.name, i, str(self.compare_auxdata), str(auxdata)))
+
         if auxdata:
             for i in auxdata:
                 if i == "oob":
@@ -512,13 +531,16 @@ class HandleData:
         self.waiter.wake()
         return
 
-def alloc_io(o, iostr, do_open = True, chunksize = 10240):
+def alloc_io(o, iostr, do_open = True, chunksize = 10240, enable_oob = False):
     """Allocate an io instance with a HandlerData handler
 
     If do_open is True (default), open it, too.
     """
     gensios_enabled.check_iostr_gensios(iostr)
     h = HandleData(o, iostr, chunksize = chunksize)
+    if enable_oob:
+        h.io.control(0, gensio.GENSIO_CONTROL_SET,
+                     gensio.GENSIO_CONTROL_ENABLE_OOB, "1")
     if (do_open):
         h.io.open_s()
     return h.io
@@ -623,18 +645,24 @@ def test_write_drain(io1, io2, data, timeout = 1000):
                          io2.handler.compared))
     return
 
-def io_close(io, timeout = 1000):
-    """close the given gensio
+def io_close(ios, timeout = 1000):
+    """close the given tuple of gensios
 
     If it does not succeed in timeout milliseconds, raise and exception.
     """
-    io.handler.close()
-    if (io.handler.wait_timeout(timeout) == 0):
-        raise Exception("%s: %s: Timed out waiting for close" %
-                        ("io_close", io.handler.name))
-    # Break all the possible circular references.
-    del io.handler.io
-    del io.handler
+    for io in ios:
+        if not io:
+            continue
+        io.handler.close()
+    for io in ios:
+        if not io:
+            continue
+        if (io.handler.wait_timeout(timeout) == 0):
+            raise Exception("%s: %s: Timed out waiting for close" %
+                            ("io_close", io.handler.name))
+        # Break all the possible circular references.
+        del io.handler.io
+        del io.handler
     return
 
 keydir = os.getenv("keydir")
@@ -668,7 +696,7 @@ def test_shutdown():
     while gensio.gensio_num_alloced() > 0:
         count += 1
         if (count > 100):
-            raise Exception("All gensios were not freed in time" % c)
+            raise Exception("All gensios were not freed in time")
         w.service(1)
     while w.service_now() == 0:
         # Give some time for everyting to clear out.
@@ -712,11 +740,14 @@ class TestAccept:
                  io1_dummy_write = None, do_close = True,
                  expected_raddr = None, expected_acc_laddr = None,
                  chunksize = 10240, get_port = True, except_on_log = False,
-                 is_sergensio = False):
+                 is_sergensio = False, enable_oob = False, timeout = 0,
+                 close_timeout = 1000):
         self.o = o
         self.io1 = None
         self.io2 = None
         self.acc = None
+        self.enable_oob = enable_oob
+        self.close_timeout = close_timeout;
 
         try:
             self.except_on_log = except_on_log
@@ -757,7 +788,7 @@ class TestAccept:
                 port = ""
             io1str = io1str + port
             io1 = alloc_io(o, io1str, do_open = False,
-                           chunksize = chunksize)
+                           chunksize = chunksize, enable_oob = enable_oob)
             self.io1 = io1
             if expected_acc_laddr:
                 expected_acc_laddr = expected_acc_laddr + port
@@ -792,7 +823,10 @@ class TestAccept:
                                                    self.io2.handler.name)) +
                                 ("Timed out waiting for dummy read at byte %d" %
                                      self.io2.handler.compared))
-            tester(self.io1, self.io2)
+            if timeout > 0:
+                tester(self.io1, self.io2, timeout=timeout)
+            else:
+                tester(self.io1, self.io2)
             if do_close:
                 self.close()
         except:
@@ -808,9 +842,7 @@ class TestAccept:
         # Close the accepter first.  Some accepters (like conacc) will
         # re-open when the child closes.
         self.acc.shutdown_s()
-        io_close(self.io1)
-        if self.io2:
-            io_close(self.io2)
+        io_close((self.io1, self.io2), timeout = self.close_timeout)
 
         # Break all the possible circular references.
         self.io1 = None
@@ -818,6 +850,9 @@ class TestAccept:
         self.acc = None
 
     def new_connection(self, acc, io):
+        if self.enable_oob:
+            io.control(0, gensio.GENSIO_CONTROL_SET,
+                       gensio.GENSIO_CONTROL_ENABLE_OOB, "1")
         HandleData(self.o, None, io = io, name = self.name)
         print("New connection " + self.name);
         self.io2 = io
@@ -836,8 +871,8 @@ class TestAccept:
     def wait_timeout(self, timeout):
         return self.waiter.wait_timeout(1, timeout)
 
-def do_test(io1, io2):
-    test_dataxfer(io1, io2, "This is a test string!")
+def do_test(io1, io2, timeout = 1000):
+    test_dataxfer(io1, io2, "This is a test string!", timeout = timeout)
     print("  Success!")
 
 def do_small_test(io1, io2, timeout=2000):
@@ -955,10 +990,7 @@ class TestAcceptConnect:
             self.io1.read_cb_enable(False)
         if self.io2:
             self.io2.read_cb_enable(False)
-        if (self.io1):
-            io_close(self.io1)
-        if self.io2:
-            io_close(self.io2)
+        io_close((self.io1, self.io2))
 
         # Break all the possible circular references.
         self.io1 = None
@@ -1035,8 +1067,7 @@ class TestConCon:
     def close(self):
         self.io1.read_cb_enable(False)
         self.io2.read_cb_enable(False)
-        io_close(self.io1)
-        io_close(self.io2)
+        io_close((self.io1, self.io2))
 
         # Break all the possible circular references.
         del self.io1
