@@ -30,7 +30,6 @@
 #include <assert.h>
 
 #include <gensio/gensio.h>
-#include <gensio/gensio_builtins.h>
 #include <gensio/gensio_list.h>
 #include <gensio/argvutils.h>
 
@@ -52,6 +51,9 @@
 #include <sys/ioctl.h>
 #include <fcntl.h>
 #include <security/pam_appl.h>
+#ifdef HAVE_SETUTXENT
+#include <utmpx.h>
+#endif
 #else
 #include <windows.h>
 #include <userenv.h>
@@ -217,6 +219,7 @@ struct auth_data {
 
     struct gdata *ginfo;
     struct gensio *rem_io;
+    struct gensio *local_io;
 
     char username[100];
     char *homedir;
@@ -415,6 +418,58 @@ write_str_to_gensio(const char *str, struct gensio *io,
     return write_buf_to_gensio(str, strlen(str), io, timeout, xlatnl);
 }
 
+static void
+write_utmp(struct auth_data *auth, bool is_login)
+{
+#ifdef HAVE_SETUTXENT
+    struct utmpx u;
+    gensiods len;
+    int err;
+    struct timeval tv;
+
+    if (!auth->interactive_login)
+	return;
+
+    memset(&u, 0, sizeof(u));
+    gettimeofday(&tv, NULL);
+    u.ut_tv.tv_sec = tv.tv_sec;
+    u.ut_tv.tv_usec = tv.tv_usec;
+    if (is_login)
+	u.ut_type = USER_PROCESS;
+    else
+	u.ut_type = DEAD_PROCESS;
+    u.ut_pid = getpid();
+    if (auth->rhost)
+	/* Don't nil terminate if full. */
+	strncpy(u.ut_host, auth->rhost, sizeof(u.ut_host));
+    strncpy(u.ut_user, auth->username, sizeof(u.ut_user));
+    len = sizeof(u.ut_line);
+    err = gensio_control(auth->local_io, 0, GENSIO_CONTROL_GET,
+			 GENSIO_CONTROL_LADDR, u.ut_line, &len);
+    if (err || len == 0) {
+	strncpy(u.ut_id, "~", sizeof(u.ut_id));
+    } else {
+	if (strncmp(u.ut_line, "/dev/", 5) == 0) {
+	    memmove(u.ut_line, u.ut_line + 5, len - 5);
+	    len -= 5;
+	    u.ut_line[len] = '\0';
+	}
+	if (len < 4)
+	    len = 0;
+	else
+	    len -= 4;
+	strncpy(u.ut_id, u.ut_line + len, sizeof(u.ut_id));
+    }
+
+    setutxent();
+    pututxline(&u);
+    endutxent();
+#ifdef WTMPX_FILE
+    updwtmpx(WTMPX_FILE, &u);
+#endif
+#endif
+}
+
 static void
 closecount_decr_ginfo(struct gdata *ginfo)
 {
@@ -563,6 +618,7 @@ close_con_info(struct per_con_info *pcinfo)
     }
 
     if (pcinfo->io2_can_close) {
+	write_utmp(auth, false);
 	pcinfo->io2_can_close = false;
 	err = gensio_close(pcinfo->io2, io_close, pcinfo);
 	if (err) {
@@ -987,14 +1043,14 @@ switch_to_user(struct auth_data *auth)
 {
     int err, err2;
 
-    err = setegid(auth->gid);
+    err = setregid(auth->gid, -1);
     if (err) {
 	log_event(LOG_ERR, "setgid failed: %s", strerror(errno));
 	return err;
     }
-    err = seteuid(auth->uid);
+    err = setreuid(auth->uid, -1);
     if (err) {
-	err2 = setegid(getgid());
+	err2 = setgid(getegid());
 	if (err2)
 	    log_event(LOG_WARNING, "err reset setgid failed: %s",
 		      strerror(errno));
@@ -1009,10 +1065,10 @@ switch_from_user(struct auth_data *auth)
 {
     int err;
 
-    err = seteuid(getuid());
+    err = setuid(geteuid());
     if (err)
 	log_event(LOG_WARNING, "reset setuid failed: %s", strerror(errno));
-    err = setegid(getgid());
+    err = setgid(getegid());
     if (err)
 	log_event(LOG_WARNING, "reset setgid failed: %s", strerror(errno));
 }
@@ -2020,7 +2076,7 @@ certauth_event(struct gensio *io, void *user_data, int event, int ierr,
 	if (!ierr) {
 	    log_event(LOG_INFO, "Accepted certificate for %s",
 		      auth->username);
-#if _WIN32
+#ifdef _WIN32
 	    /*
 	     * On windows, we cache the password in .gtlssh.  Not
 	     * ideal, but Windows logons don't work very well if you
@@ -2082,7 +2138,8 @@ certauth_event(struct gensio *io, void *user_data, int event, int ierr,
     }
 }
 
-static void
+/* Returns true if successful, false if not. */
+static bool
 new_rem_io(struct gensio *io, struct auth_data *auth)
 {
     struct gensio_os_funcs *o = auth->ginfo->o;
@@ -2098,6 +2155,7 @@ new_rem_io(struct gensio *io, struct auth_data *auth)
     char **env = NULL;
     unsigned int env_len = 0;
     const char **penv2 = NULL;
+    bool rv = false;
 
     len = 0;
     err = gensio_control(io, 0, GENSIO_CONTROL_GET, GENSIO_CONTROL_SERVICE,
@@ -2146,7 +2204,11 @@ new_rem_io(struct gensio *io, struct auth_data *auth)
 	err = get_vals_from_service(o, &env, &env_len, str, len);
 	if (err)
 	    goto out_bad_vals;
+#ifdef _WIN32
 	s = gensio_alloc_sprintf(o, "pty,%s -i", auth->ushell);
+#else
+	s = gensio_alloc_sprintf(o, "pty,-%s -i", auth->ushell);
+#endif
 	login = true;
 	do_chdir = true;
     } else if (strstartswith(service, "tcp,") ||
@@ -2323,6 +2385,7 @@ new_rem_io(struct gensio *io, struct auth_data *auth)
 	goto out_err;
     }
     pcinfo->io2 = pty_io;
+    auth->local_io = pty_io;
 
     if (do_chdir) {
 	err = gensio_control(pty_io, 0, GENSIO_CONTROL_SET,
@@ -2385,6 +2448,7 @@ new_rem_io(struct gensio *io, struct auth_data *auth)
     }
 
     io = NULL;
+    rv = true;
 
     goto out_free;
 
@@ -2405,6 +2469,7 @@ new_rem_io(struct gensio *io, struct auth_data *auth)
 	o->free(o, service);
     if (s)
 	o->free(o, s);
+    return rv;
 }
 
 static int
@@ -2453,7 +2518,8 @@ open_mux(struct gensio **io, struct auth_data *auth, struct gensio **nio)
 				       "readbuf=1048128",
 				       NULL };
 
-    err = mux_gensio_alloc(*io, isclient, o, mux_event, auth, &mux_io);
+    err = gensio_filter_alloc("mux", *io, isclient, o, mux_event, auth,
+			      &mux_io);
     if (err) {
 	log_event(LOG_ERR, "Unable to allocate mux gensio: %s",
 		  gensio_err_to_str(err));
@@ -2527,7 +2593,7 @@ handle_new(struct gensio *net_io)
     auth->pam_conv.appdata_ptr = auth;
 #endif
 
-    err = ssl_gensio_alloc(net_io, ssl_args, o, NULL, NULL, &ssl_io);
+    err = gensio_filter_alloc("ssl", net_io, ssl_args, o, NULL, NULL, &ssl_io);
     if (err) {
 	log_event(LOG_ERR, "Unable to allocate SSL gensio: %s",
 		  gensio_err_to_str(err));
@@ -2548,8 +2614,8 @@ handle_new(struct gensio *net_io)
     if (do_2fa)
 	certauth_args[i++] = "enable-2fa";
 
-    err = certauth_gensio_alloc(ssl_io, certauth_args, o,
-				certauth_event, auth, &certauth_io);
+    err = gensio_filter_alloc("certauth", ssl_io, certauth_args, o,
+			      certauth_event, auth, &certauth_io);
     if (err) {
 	log_event(LOG_ERR, "Unable to allocate certauth gensio: %s",
 		  gensio_err_to_str(err));
@@ -2662,7 +2728,8 @@ handle_new(struct gensio *net_io)
     ginfo_unlock(ginfo);
 
     start_local_ports(auth->locport, top_io);
-    new_rem_io(top_io, auth);
+    if (new_rem_io(top_io, auth))
+	write_utmp(auth, true);
 
     return;
 
@@ -2940,7 +3007,7 @@ help(int err)
     printf("  --oneshot - Do not fork new connections, do one and exit.\n");
     printf("  --nodaemon - Do not daemonize.\n");
     printf("  --nointeractive - Do not do interactive login queries.\n");
-    printf("  --nosctp - Disable SCTP support.\n");
+    printf("  --sctp - Enable SCTP support.\n");
     printf("  --notcp - Disable TCP support.\n");
     printf("  --other_acc <accepter> - Allows the user to specify the\n");
     printf("     accepter used by gtlsshd, in addition to sctp and tcp.\n");
@@ -3003,7 +3070,7 @@ main(int argc, char *argv[])
     const char *configfile = default_configfile;
     unsigned int port = 852;
     char *s;
-    bool notcp = false, nosctp = false;
+    bool notcp = false, sctp = false;
     bool daemonize = true;
     const char *iptype = ""; /* Try both IPv4 and IPv6 by default. */
     const char *other_acc_str = NULL;
@@ -3035,8 +3102,8 @@ main(int argc, char *argv[])
 	    ;
 	else if ((rv = cmparg(argc, argv, &arg, NULL, "--notcp", NULL)))
 	    notcp = true;
-	else if ((rv = cmparg(argc, argv, &arg, NULL, "--nosctp", NULL)))
-	    nosctp = true;
+	else if ((rv = cmparg(argc, argv, &arg, NULL, "--sctp", NULL)))
+	    sctp = true;
 	else if ((rv = cmparg(argc, argv, &arg, NULL, "--other_acc",
 			      &other_acc_str)))
 	    ;
@@ -3083,7 +3150,7 @@ main(int argc, char *argv[])
     start_log(debug);
     log_event(LOG_NOTICE, "gtlsshd startup");
 
-    if (nosctp && notcp) {
+    if (!sctp && notcp) {
 	log_event(LOG_ERR, "You cannot disable both TCP and SCTP\n");
 	exit(1);
     }
@@ -3168,7 +3235,7 @@ main(int argc, char *argv[])
 	}
     }
 
-    if (!nosctp) {
+    if (sctp) {
 	s = gensio_alloc_sprintf(o, "sctp(readbuf=20000),%s%d", iptype, port);
 	if (!s) {
 	    log_event(LOG_ERR, "Could not allocate sctp descriptor\n");
